@@ -12,6 +12,7 @@
 #include <pobl/bl_mem.h>
 #include <pobl/bl_str.h> /* bl_str_alloca_dup */
 #include <pobl/bl_path.h>
+#include <pobl/bl_dialog.h>
 
 #include "../ui_window.h"
 #include "../../common/c_animgif.c"
@@ -255,6 +256,24 @@ static JNIEnv *get_jni_env(JavaVM *vm) {
   }
 }
 
+static int dialog_cb(bl_dialog_style_t style, const char *msg) {
+  JNIEnv *env;
+
+  if (style == BL_DIALOG_ALERT &&
+      (env = get_jni_env(_display.app->activity->vm))) {
+    jobject this = _display.app->activity->clazz;
+    (*env)->CallVoidMethod(env, this,
+                           (*env)->GetMethodID(env, (*env)->GetObjectClass(env, this),
+                                               "showMessage",
+                                               "(Ljava/lang/String;)V"),
+                           (*env)->NewStringUTF(env, msg));
+
+    return 1;
+  } else {
+    return -1;
+  }
+}
+
 static void show_soft_input(JavaVM *vm) {
   JNIEnv *env;
 
@@ -565,8 +584,11 @@ static void on_app_cmd(struct android_app *app, int32_t cmd) {
 /* --- global functions --- */
 
 ui_display_t *ui_display_open(char *disp_name, u_int depth) {
-  if (!(_disp.name = getenv("DISPLAY"))) {
+  if (!_disp.name && !(_disp.name = getenv("DISPLAY"))) {
     _disp.name = ":0.0";
+
+    /* Callback should be set before bl_dialog() is called. */
+    bl_dialog_set_callback(dialog_cb);
   }
   _disp.display = &_display;
 
@@ -977,7 +999,7 @@ void ui_display_send_text_selection(u_char *sel_data, size_t sel_len) {
                          (*env)->NewStringUTF(env, sel_data));
 }
 
-void ui_display_show_dialog(char *server) {
+void ui_display_show_dialog(char *server, char *privkey) {
   char *user;
   char *host;
   char *port;
@@ -994,16 +1016,26 @@ void ui_display_show_dialog(char *server) {
     user = host = port = encoding = NULL;
   }
 
+  if (!privkey) {
+    char *home;
+
+    if ((home = bl_get_home_dir()) && (privkey = alloca(strlen(home) + 15 + 1))) {
+      sprintf(privkey, "%s/.mlterm/id_rsa", home);
+    }
+  }
+
   this = _display.app->activity->clazz;
   (*env)->CallVoidMethod(env, this,
                          (*env)->GetMethodID(env, (*env)->GetObjectClass(env, this),
-                                             "showDialog",
+                                             "showConnectDialog",
                                              "(Ljava/lang/String;Ljava/lang/String;"
-                                             "Ljava/lang/String;Ljava/lang/String;)V"),
+                                             "Ljava/lang/String;Ljava/lang/String;"
+                                             "Ljava/lang/String;)V"),
                          user ? (*env)->NewStringUTF(env, user) : NULL,
                          host ? (*env)->NewStringUTF(env, host) : NULL,
                          port ? (*env)->NewStringUTF(env, port) : NULL,
-                         encoding ? (*env)->NewStringUTF(env, encoding) : NULL);
+                         encoding ? (*env)->NewStringUTF(env, encoding) : NULL,
+                         privkey ? (*env)->NewStringUTF(env, privkey) : NULL);
 }
 
 /* Called in the main thread (not in the native activity thread) */
