@@ -33,6 +33,7 @@ import android.widget.EditText;
 import android.widget.TextView;
 import android.widget.LinearLayout;
 import android.widget.AdapterView.AdapterContextMenuInfo;
+import android.widget.Toast;
 import android.content.DialogInterface;
 import android.app.AlertDialog;
 import android.app.Dialog;
@@ -50,11 +51,12 @@ public class MLActivity extends NativeActivity {
   private native String convertToTmpPath(String path);
   private native void splitAnimationGif(String path);
   private native void dialogOkClicked(String user, String serv, String port, String encoding,
-                                      String pass, String cmd);
+                                      String pass, String cmd, String privkey);
   private native void updateScreen();
-  private native void execCommand(int cmd);
+  private native void execCommand(String cmd);
   private native void resumeNative();
   private native void pauseNative();
+  private native boolean isSSH();
 
   private String keyString;
   private View contentView;
@@ -284,18 +286,22 @@ public class MLActivity extends NativeActivity {
   private final int MENU_PREVPTY_ID = 7;
   private final int MENU_UPDATESCREEN_ID = 8;
   private final int MENU_CLOSESCREEN_ID = 9;
-  private final int MENU_CONFIG_ID = 10;
-  private final int MENU_CANCEL_ID = 11;
+  private final int MENU_SCP_ID = 10;
+  private final int MENU_CONFIG_ID = 11;
+  private final int MENU_CANCEL_ID = 12;
 
   @Override
   public void onCreateContextMenu(ContextMenu menu, View view,
-                                     ContextMenu.ContextMenuInfo info) {
+                                  ContextMenu.ContextMenuInfo info) {
     super.onCreateContextMenu(menu, view, info);
     /* menu.setHeaderTitle("Menu"); */
     menu.add(0, MENU_PASTE_ID, 0, "Paste from clipboard");
     menu.add(0, MENU_NEWSCREEN_ID, 0, "Open new screen");
     menu.add(0, MENU_LOCALPTY_ID, 0, "Open local pty");
     menu.add(0, MENU_SSH_ID, 0, "Connect to SSH server");
+    if (isSSH()) {
+      menu.add(0, MENU_SCP_ID, 0, "SCP");
+    }
     menu.add(0, MENU_VSPLIT_ID, 0, "Split screen vertically");
     menu.add(0, MENU_HSPLIT_ID, 0, "Split screen horizontally");
     menu.add(0, MENU_NEXTPTY_ID, 0, "Next pty");
@@ -313,31 +319,34 @@ public class MLActivity extends NativeActivity {
         getTextFromClipboard();
         return true;
       case MENU_NEWSCREEN_ID:
-        execCommand(0);
+        execCommand("open_pty");
         return true;
       case MENU_LOCALPTY_ID:
-        execCommand(3);
+        execCommand("mlclientx --serv=");
         return true;
       case MENU_SSH_ID:
-        execCommand(7);
+        execCommand("ssh");
         return true;
       case MENU_VSPLIT_ID:
-        execCommand(1);
+        execCommand("vsplit_screen");
         return true;
       case MENU_HSPLIT_ID:
-        execCommand(2);
+        execCommand("hsplit_screen");
         return true;
       case MENU_NEXTPTY_ID:
-        execCommand(4);
+        execCommand("next_pty");
         return true;
       case MENU_PREVPTY_ID:
-        execCommand(5);
+        execCommand("prev_pty");
         return true;
       case MENU_UPDATESCREEN_ID:
         updateScreen();
         return true;
       case MENU_CLOSESCREEN_ID:
-        execCommand(6);
+        execCommand("close_screen");
+        return true;
+      case MENU_SCP_ID:
+        showDialog(2); /* SCP dialog */
         return true;
       case MENU_CONFIG_ID:
         Intent intent = new Intent(this, MLPreferenceActivity.class);
@@ -456,41 +465,87 @@ public class MLActivity extends NativeActivity {
   private EditText pass_edit;
   private EditText encoding_edit;
   private EditText cmd_edit;
+  private EditText privkey_edit;
   private LinearLayout dialogLayout;
 
   @Override protected Dialog onCreateDialog(int id) {
-    Dialog dialog = new AlertDialog.Builder(this)
-      .setIcon(android.R.drawable.ic_dialog_info)
-      .setTitle("Connect to SSH Server")
-      .setView(dialogLayout)
-      .setPositiveButton("Connect", new DialogInterface.OnClickListener() {
-          public void onClick(DialogInterface dialog, int whichButton) {
-            dialogOkClicked(user_edit.getText().toString(),
-                            serv_edit.getText().toString(),
-                            port_edit.getText().toString(),
-                            encoding_edit.getText().toString(),
-                            pass_edit.getText().toString(),
-                            cmd_edit.getText().toString());
-          }
-        })
-      .setNegativeButton("Local", new DialogInterface.OnClickListener() {
-          public void onClick(DialogInterface dialog, int whichButton) {} })
-      .create();
+    if (id == 1) {
+      Dialog dialog = new AlertDialog.Builder(this)
+        .setIcon(android.R.drawable.ic_dialog_info)
+        .setTitle("Connect to SSH Server")
+        .setView(dialogLayout)
+        .setPositiveButton("Connect", new DialogInterface.OnClickListener() {
+            public void onClick(DialogInterface dialog, int whichButton) {
+              dialogOkClicked(user_edit.getText().toString(),
+                              serv_edit.getText().toString(),
+                              port_edit.getText().toString(),
+                              encoding_edit.getText().toString(),
+                              pass_edit.getText().toString(),
+                              cmd_edit.getText().toString(),
+                              privkey_edit.getText().toString());
+            }
+          })
+        .setNegativeButton("Local", null)
+        .create();
 
-    dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
-        public void onDismiss(DialogInterface dialog) {
-          if (nativeThread != null) {
-            synchronized(nativeThread) {
-              nativeThread.notifyAll();
+      dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
+          public void onDismiss(DialogInterface dialog) {
+            if (nativeThread != null) {
+              synchronized(nativeThread) {
+                nativeThread.notifyAll();
+              }
             }
           }
-        }
-      });
+        });
 
-    return dialog;
+      return dialog;
+    } else if (id == 2) {
+      final EditText remote_edit;
+      final EditText local_edit;
+
+      LinearLayout layout = new LinearLayout(this);
+      layout.setOrientation(LinearLayout.VERTICAL);
+
+      LinearLayout.LayoutParams params =
+        new LinearLayout.LayoutParams(LinearLayout.LayoutParams.FILL_PARENT,
+                                      LinearLayout.LayoutParams.FILL_PARENT);
+
+      remote_edit = new EditText(this);
+      remote_edit.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_URI);
+      layout.addView(makeTextEntry("Remote", remote_edit), params);
+
+      local_edit = new EditText(this);
+      local_edit.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_URI);
+      layout.addView(makeTextEntry("Local ", local_edit), params);
+
+      Dialog dialog = new AlertDialog.Builder(this)
+        .setIcon(android.R.drawable.ic_dialog_info)
+        .setTitle("Source file path / Dest dir path")
+        .setView(layout)
+        .setPositiveButton("Send", new DialogInterface.OnClickListener() {
+            public void onClick(DialogInterface dialog, int whichButton) {
+              execCommand("scp \"local:" + local_edit.getText().toString() + "\" \"remote:" +
+                          remote_edit.getText().toString() + "\"");
+            }
+          })
+        .setNeutralButton("Receive", new DialogInterface.OnClickListener() {
+            public void onClick(DialogInterface dialog, int whichButton) {
+              execCommand("scp \"remote:" + remote_edit.getText().toString() + "\" \"local:" +
+                          local_edit.getText().toString() + "\"");
+            }
+          })
+        .setNegativeButton("Cancel", null)
+        .create();
+
+      return dialog;
+    } else {
+      return null;
+    }
   }
 
-  private void showDialog(String user, String serv, String port, String encoding) {
+  /* Called from native activity thread */
+  private void showConnectDialog(String user, String serv, String port, String encoding,
+                                 String privkey) {
     nativeThread = Thread.currentThread();
 
     dialogLayout = new LinearLayout(this);
@@ -526,6 +581,10 @@ public class MLActivity extends NativeActivity {
     cmd_edit.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_URI);
     dialogLayout.addView(makeTextEntry("Exec Cmd ", cmd_edit), params);
 
+    privkey_edit = new EditText(this);
+    privkey_edit.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_URI);
+    dialogLayout.addView(makeTextEntry("Priv key ", privkey_edit), params);
+
     if (serv != null) {
       serv_edit.setText(serv, TextView.BufferType.NORMAL);
     }
@@ -542,24 +601,37 @@ public class MLActivity extends NativeActivity {
       encoding_edit.setText(encoding, TextView.BufferType.NORMAL);
     }
 
-    if (nativeThread != null) {
-      synchronized(nativeThread) {
-        runOnUiThread(new Runnable() {
-            @Override public void run() {
-              showDialog(1);
-            }
-          });
-
-        try {
-          nativeThread.wait();
-        } catch (InterruptedException e) {}
-      }
-
-      nativeThread = null;
+    if (privkey != null) {
+      privkey_edit.setText(privkey, TextView.BufferType.NORMAL);
     }
 
-    serv_edit = port_edit = user_edit = pass_edit = encoding_edit = null;
+    synchronized(nativeThread) {
+      runOnUiThread(new Runnable() {
+          @Override public void run() {
+            showDialog(1);
+          }
+        });
 
+      try {
+        nativeThread.wait();
+      } catch (InterruptedException e) {}
+    }
+
+    nativeThread = null;
     removeDialog(1);
+
+    serv_edit = port_edit = user_edit = pass_edit = encoding_edit = privkey_edit = null;
+  }
+
+  /* Called from native activity thread or UI thread (from context menu) */
+  private void showMessage(String message) {
+    final Context ctx = this;
+    final String msg = message;
+
+    runOnUiThread(new Runnable() {
+        @Override public void run() {
+          Toast.makeText(ctx, msg, Toast.LENGTH_LONG).show();
+        }
+      });
   }
 }
