@@ -157,23 +157,23 @@ static void preedit(im_wnn_t *wnn, char *preedit,
   } else {
     ef_char_t ch;
     vt_char_t *p;
-    u_int num_of_chars;
+    u_int num_chars;
     u_int len;
     u_char *tmp = NULL;
     size_t pos_len;
 
     wnn->im.preedit.cursor_offset = rev_pos;
 
-    num_of_chars = 0;
+    num_chars = 0;
     (*parser_wchar->init)(parser_wchar);
     (*parser_wchar->set_str)(parser_wchar, preedit, preedit_len);
     while ((*parser_wchar->next_char)(parser_wchar, &ch)) {
-      num_of_chars++;
+      num_chars++;
     }
 
     pos_len = strlen(pos);
 
-    if ((p = realloc(wnn->im.preedit.chars, sizeof(vt_char_t) * (num_of_chars + pos_len))) ==
+    if ((p = realloc(wnn->im.preedit.chars, sizeof(vt_char_t) * (num_chars + pos_len))) ==
         NULL) {
       return;
     }
@@ -185,7 +185,7 @@ static void preedit(im_wnn_t *wnn, char *preedit,
     }
 
     (*syms->vt_str_init)(wnn->im.preedit.chars = p,
-                         wnn->im.preedit.num_of_chars = (num_of_chars + pos_len));
+                         wnn->im.preedit.num_chars = (num_chars + pos_len));
     wnn->im.preedit.filled_len = 0;
 
     (*wnn->parser_term->init)(wnn->parser_term);
@@ -211,7 +211,7 @@ static void preedit(im_wnn_t *wnn, char *preedit,
         is_comb = 1;
 
         if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb,
-                                     VT_FG_COLOR, VT_BG_COLOR, 0, 0, 1, 0, 0, 0)) {
+                                     VT_FG_COLOR, VT_BG_COLOR, 0, 0, LS_UNDERLINE_SINGLE, 0, 0)) {
           continue;
         }
 
@@ -225,10 +225,10 @@ static void preedit(im_wnn_t *wnn, char *preedit,
       if (wnn->im.preedit.cursor_offset <= wnn->im.preedit.filled_len &&
           wnn->im.preedit.filled_len < wnn->im.preedit.cursor_offset + rev_len) {
         (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, VT_BG_COLOR,
-                             VT_FG_COLOR, 0, 0, 1, 0, 0, 0);
+                             VT_FG_COLOR, 0, 0, LS_UNDERLINE_SINGLE, 0, 0);
       } else {
         (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, VT_FG_COLOR,
-                             VT_BG_COLOR, 0, 0, 1, 0, 0, 0);
+                             VT_BG_COLOR, 0, 0, LS_UNDERLINE_SINGLE, 0, 0);
       }
 
       p++;
@@ -236,8 +236,8 @@ static void preedit(im_wnn_t *wnn, char *preedit,
     }
 
     for (; pos_len > 0; pos_len--) {
-      (*syms->vt_char_set)(p++, *(pos++), US_ASCII, 0, 0, VT_FG_COLOR, VT_BG_COLOR, 0, 0, 1,
-                           0, 0, 0);
+      (*syms->vt_char_set)(p++, *(pos++), US_ASCII, 0, 0, VT_FG_COLOR, VT_BG_COLOR, 0, 0,
+                           LS_UNDERLINE_SINGLE, 0, 0);
       wnn->im.preedit.filled_len++;
     }
 
@@ -580,7 +580,7 @@ static int fix(im_wnn_t *wnn) {
  * methods of ui_im_t
  */
 
-static int delete (ui_im_t *im) {
+static void delete(ui_im_t *im) {
   im_wnn_t *wnn;
   struct wnn_buf *buf;
 
@@ -608,8 +608,6 @@ static int delete (ui_im_t *im) {
     (*parser_wchar->delete)(parser_wchar);
     parser_wchar = NULL;
   }
-
-  return ref_count;
 }
 
 static int switch_mode(ui_im_t *im) {
@@ -833,7 +831,7 @@ static void unfocused(ui_im_t *im) {
 /* --- global functions --- */
 
 ui_im_t *im_wnn_new(u_int64_t magic, vt_char_encoding_t term_encoding,
-                    ui_im_export_syms_t *export_syms, char *engine, u_int mod_ignore_mask) {
+                    ui_im_export_syms_t *export_syms, char *server, u_int mod_ignore_mask) {
   im_wnn_t *wnn;
   struct wnn_buf *buf;
 
@@ -867,7 +865,7 @@ ui_im_t *im_wnn_new(u_int64_t magic, vt_char_encoding_t term_encoding,
     goto error;
   }
 
-  if (!(buf = jcOpen(NULL, "", 0, "", bl_msg_printf, bl_msg_printf, 0))) {
+  if (!(buf = jcOpen(server, "", 0, "", bl_msg_printf, bl_msg_printf, 0))) {
 #ifdef DEBUG
     bl_debug_printf(BL_DEBUG_TAG " jcOpen failed.\n");
 #endif
@@ -930,7 +928,7 @@ im_info_t *im_wnn_get_info(char *locale, char *encoding) {
   if ((result = malloc(sizeof(im_info_t)))) {
     result->id = strdup("wnn");
     result->name = strdup("Wnn");
-    result->num_of_args = 0;
+    result->num_args = 0;
     result->args = NULL;
     result->readable_args = NULL;
   }
