@@ -7,30 +7,14 @@
 
 #include "mlterm_MLTermPty.h"
 
-#include <unistd.h> /* getcwd */
-#include <pobl/bl_locale.h>
 #include <pobl/bl_debug.h>
-#include <pobl/bl_sig_child.h>
 #include <pobl/bl_mem.h> /* alloca */
 #include <pobl/bl_str.h> /* bl_str_alloca_dup */
-#include <pobl/bl_conf.h>
-#include <pobl/bl_conf_io.h>
-#include <pobl/bl_path.h>   /* bl_basename */
 #include <pobl/bl_unistd.h> /* bl_setenv */
-#include <pobl/bl_args.h>   /* _bl_arg_str_to_array */
 #include <pobl/bl_dialog.h>
 #include <mef/ef_utf16_conv.h>
 #include <vt_str_parser.h>
-#include <vt_term_manager.h>
-#include <../main/version.h>
-
-#if defined(USE_WIN32API)
-#define CONFIG_PATH "."
-#elif defined(SYSCONFDIR)
-#define CONFIG_PATH SYSCONFDIR
-#else
-#define CONFIG_PATH "/etc"
-#endif
+#include <mlterm.h>
 
 /* Same as those defined in SWT.java */
 #define AltMask (1 << 16)
@@ -41,10 +25,6 @@
 #define TUNEUP_HACK
 #endif
 
-#if 0
-#define USE_LOCAL_ECHO_BY_DEFAULT
-#endif
-
 #if 0
 #define __DEBUG
 #endif
@@ -74,10 +54,6 @@ static ef_conv_t *utf16_conv;
 static char *plink;
 #endif
 
-#ifdef USE_LIBSSH2
-static u_int keepalive_interval;
-#endif
-
 /* --- static functions --- */
 
 static void pty_closed(void *p) {
@@ -98,7 +74,7 @@ static int dialog_callback(bl_dialog_style_t style, const char *msg) {
   jclass class;
   static jmethodID mid;
 
-  if (style != BL_DIALOG_OKCANCEL || !env_for_dialog) {
+  if (!env_for_dialog) {
     return -1;
   }
 
@@ -107,12 +83,13 @@ static int dialog_callback(bl_dialog_style_t style, const char *msg) {
 
   if (!mid) {
     mid = (*env_for_dialog)
-              ->GetStaticMethodID(env_for_dialog, class, "show", "(Ljava/lang/String;)Z");
+              ->GetStaticMethodID(env_for_dialog, class, "show", "(Ljava/lang/String;Z)Z");
   }
 
   if ((*env_for_dialog)
           ->CallStaticObjectMethod(env_for_dialog, class, mid,
-                                   (*env_for_dialog)->NewStringUTF(env_for_dialog, msg))) {
+                                   (*env_for_dialog)->NewStringUTF(env_for_dialog, msg),
+                                   style == BL_DIALOG_OKCANCEL ? JNI_TRUE : JNI_FALSE)) {
     return 1;
   } else {
     return 0;
@@ -133,12 +110,6 @@ static void set_config(void *p, char *dev, char *key, char *value) {
     if ((encoding = vt_get_char_encoding(value)) != VT_UNKNOWN_ENCODING) {
       vt_term_change_encoding(nativeObj->term, encoding);
     }
-  } else if (strcmp(key, "use_multi_column_char") == 0) {
-    int flag;
-
-    if ((flag = true_or_false(value)) != -1) {
-      vt_term_set_use_multi_col_char(nativeObj->term, flag);
-    }
   }
 }
 
@@ -163,13 +134,7 @@ static void get_config(void *p, char *dev, char *key, int to_menu) {
 
   value = NULL;
 
-  if (strcmp(key, "use_multi_column_char") == 0) {
-    if (vt_term_is_using_multi_col_char(term)) {
-      value = "true";
-    } else {
-      value = "false";
-    }
-  } else if (strcmp(key, "pty_list") == 0) {
+  if (strcmp(key, "pty_list") == 0) {
     value = vt_get_pty_list();
   }
 
@@ -188,7 +153,7 @@ static int exec_cmd(void *p, char *cmd) {
   nativeObj = p;
 
   if (strncmp(cmd, "browser", 7) != 0) {
-    return 0;
+    return vt_term_exec_cmd(nativeObj->term, cmd);
   }
 
   jstr_cmd = (*nativeObj->env)->NewStringUTF(nativeObj->env, cmd);
@@ -406,69 +371,14 @@ Java_mlterm_MLTermPty_nativeOpen(JNIEnv *env, jobject obj, jstring jstr_host,
                                  jstring jstr_pass, /* can be NULL */
                                  jint cols, jint rows, jstring jstr_encoding, jarray jarray_argv) {
   native_obj_t *nativeObj;
-  vt_char_encoding_t encoding;
-  int is_auto_encoding;
+  char *encoding_str;
   char **argv;
   char *host;
   char *pass;
   char *envv[4];
   char *cmd_path;
-  int ret;
-
-  static u_int tab_size;
-  static vt_char_encoding_t encoding_default;
-  static int is_auto_encoding_default;
-  static vt_unicode_policy_t unicode_policy;
-  static u_int col_size_a;
-  static int use_char_combining;
-  static int use_multi_col_char;
-  static int use_login_shell;
-  static int logging_vt_seq;
-  static int use_local_echo;
-  static int use_auto_detect;
-  static int use_ansi_colors;
-  static char *public_key;
-  static char *private_key;
-  static char **default_argv;
-  static char *default_cmd_path;
-
-#if defined(USE_WIN32API) && defined(USE_LIBSSH2)
-  static int wsa_inited = 0;
-
-  if (!wsa_inited) {
-    WSADATA wsadata;
-    WSAStartup(MAKEWORD(2, 0), &wsadata);
-    wsa_inited = 1;
-
-    /*
-     * Prevent vt_pty_ssh from spawning a thread to watch pty.
-     * If both vt_pty_ssh and MLTerm.java spawn threads to watch ptys,
-     * problems will happen in reloading (that is, finalizing and
-     * starting simultaneously) an applet page (mltermlet.html)
-     * in a web browser.
-     *
-     * This hack doesn't make sense to vt_pty_pipewin32.c.
-     * In the first place, if vt_pty_pipewin32 is used,
-     * Java_mlterm_MLTermPty_waitForReading() returns JNI_FALSE
-     * and PtyWather thread of MLTerm.java immediately exits.
-     * So threads in vt_pty_pipewin32 should not be prevented.
-     */
-    CreateEvent(NULL, FALSE, FALSE, "PTY_READ_READY");
-  }
-#endif
 
   if (!str_parser) {
-    bl_conf_t *conf;
-
-    bl_init_prog("mlterm", "3.8.4");
-    bl_set_sys_conf_dir(CONFIG_PATH);
-    bl_locale_init("");
-    bl_sig_child_init();
-#ifdef USE_LIBSSH2
-    bl_dialog_set_callback(dialog_callback);
-#endif
-    vt_term_manager_init(1);
-
     str_parser = vt_str_parser_new();
     utf8_parser = vt_char_encoding_parser_new(VT_UTF8);
 #ifdef WORDS_BIGENDIAN
@@ -477,227 +387,21 @@ Java_mlterm_MLTermPty_nativeOpen(JNIEnv *env, jobject obj, jstring jstr_host,
     utf16_conv = ef_utf16le_conv_new();
 #endif
 
-    tab_size = 8;
-    encoding_default = vt_get_char_encoding("auto");
-    is_auto_encoding_default = 1;
-
-    if (strcmp(bl_get_lang(), "ja") == 0) {
-      col_size_a = 2;
-    } else {
-      col_size_a = 1;
-    }
-
-    use_char_combining = 1;
-    use_multi_col_char = 1;
-#ifdef USE_LOCAL_ECHO_BY_DEFAULT
-    use_local_echo = 1;
-#endif
-    use_ansi_colors = 1;
-
-    if ((conf = bl_conf_new())) {
-      char *rcpath;
-      char *value;
-
-      if ((rcpath = bl_get_sys_rc_path("mlterm/main"))) {
-        bl_conf_read(conf, rcpath);
-        free(rcpath);
-      }
-
-      if ((rcpath = bl_get_user_rc_path("mlterm/main"))) {
-        bl_conf_read(conf, rcpath);
-        free(rcpath);
-      }
-
-      if ((value = bl_conf_get_value(conf, "logging_msg")) && strcmp(value, "false") == 0) {
-        bl_set_msg_log_file_name(NULL);
-      } else {
-        bl_set_msg_log_file_name("mlterm/msg.log");
-      }
-
-      if ((value = bl_conf_get_value(conf, "logging_vt_seq")) && strcmp(value, "true") == 0) {
-        logging_vt_seq = 1;
-      }
-
-      if ((value = bl_conf_get_value(conf, "tabsize"))) {
-        bl_str_to_uint(&tab_size, value);
-      }
-
-      if ((value = bl_conf_get_value(conf, "encoding"))) {
-        vt_char_encoding_t e;
-        if ((e = vt_get_char_encoding(value)) != VT_UNKNOWN_ENCODING) {
-          encoding_default = e;
-
-          if (strcmp(value, "auto") == 0) {
-            is_auto_encoding_default = 1;
-          } else {
-            is_auto_encoding_default = 0;
-          }
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "not_use_unicode_font"))) {
-        if (strcmp(value, "true") == 0) {
-          unicode_policy = NOT_USE_UNICODE_FONT;
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "only_use_unicode_font"))) {
-        if (strcmp(value, "true") == 0) {
-          if (unicode_policy == NOT_USE_UNICODE_FONT) {
-            unicode_policy = 0;
-          } else {
-            unicode_policy = ONLY_USE_UNICODE_FONT;
-          }
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "col_size_of_width_a"))) {
-        bl_str_to_uint(&col_size_a, value);
-      }
-
-      if ((value = bl_conf_get_value(conf, "use_combining"))) {
-        if (strcmp(value, "false") == 0) {
-          use_char_combining = 0;
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "use_muti_col_char"))) {
-        if (strcmp(value, "false") == 0) {
-          use_multi_col_char = 0;
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "use_login_shell"))) {
-        if (strcmp(value, "true") == 0) {
-          use_login_shell = 1;
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "use_local_echo"))) {
-#ifdef USE_LOCAL_ECHO_BY_DEFAULT
-        if (strcmp(value, "false") == 0) {
-          use_local_echo = 0;
-        }
-#else
-        if (strcmp(value, "true") == 0) {
-          use_local_echo = 1;
-        }
-#endif
-      }
-
-      if ((value = bl_conf_get_value(conf, "use_alt_buffer"))) {
-        if (strcmp(value, "false") == 0) {
-          vt_set_use_alt_buffer(0);
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "use_ansi_colors"))) {
-        if (strcmp(value, "false") == 0) {
-          use_ansi_colors = 0;
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "auto_detect_encodings"))) {
-        vt_set_auto_detect_encodings(value);
-      }
-
-      if ((value = bl_conf_get_value(conf, "use_auto_detect"))) {
-        if (strcmp(value, "true") == 0) {
-          use_auto_detect = 1;
-        }
-      }
-
 #ifdef USE_LIBSSH2
-      if ((value = bl_conf_get_value(conf, "ssh_public_key"))) {
-        public_key = strdup(value);
-      }
-
-      if ((value = bl_conf_get_value(conf, "ssh_private_key"))) {
-        private_key = strdup(value);
-      }
-
-      if ((value = bl_conf_get_value(conf, "cipher_list"))) {
-        vt_pty_ssh_set_cipher_list(strdup(value));
-      }
-
-      if ((value = bl_conf_get_value(conf, "ssh_keepalive_interval"))) {
-        if (bl_str_to_uint(&keepalive_interval, value) && keepalive_interval > 0) {
-          vt_pty_ssh_set_keepalive_interval(keepalive_interval);
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "ssh_x11_forwarding"))) {
-        if (strcmp(value, "true") == 0) {
-          vt_pty_ssh_set_use_x11_forwarding(NULL, 1);
-        }
-      }
-
-      if ((value = bl_conf_get_value(conf, "allow_scp"))) {
-        if (strcmp(value, "true") == 0) {
-          vt_set_use_scp_full(1);
-        }
-      }
+    bl_dialog_set_callback(dialog_callback);
 #endif
-
-#if 0
-      /* XXX How to get password ? */
-      if ((value = bl_conf_get_value(conf, "default_server"))) {
-        default_server = strdup(value);
-      }
-#endif
-
-      if (((value = bl_conf_get_value(conf, "exec_cmd"))
-#if defined(USE_WIN32API) && !defined(USE_LIBSSH2)
-           || (value = plink)
-#endif
-               ) &&
-          (default_argv = malloc(sizeof(char *) * bl_count_char_in_str(value, ' ') + 2))) {
-        int argc;
-
-        _bl_arg_str_to_array(default_argv, &argc, strdup(value));
-        default_cmd_path = default_argv[0];
-      }
-
-      bl_conf_delete(conf);
-    }
-
-    unicode_policy |= ONLY_USE_UNICODE_BOXDRAW_FONT;
   }
 
   if (!(nativeObj = calloc(sizeof(native_obj_t), 1))) {
     return 0;
   }
 
-  encoding = encoding_default;
-  is_auto_encoding = is_auto_encoding_default;
-
   if (jstr_encoding) {
-    char *p;
-    vt_char_encoding_t e;
-
-    p = (*env)->GetStringUTFChars(env, jstr_encoding, NULL);
-
-    if ((e = vt_get_char_encoding(p)) != VT_UNKNOWN_ENCODING) {
-      encoding = e;
-
-      if (strcmp(p, "auto") == 0) {
-        is_auto_encoding = 1;
-      } else {
-        is_auto_encoding = 0;
-      }
-    }
-
+    char *p = (*env)->GetStringUTFChars(env, jstr_encoding, NULL);
+    encoding_str = bl_str_alloca_dup(p);
     (*env)->ReleaseStringUTFChars(env, jstr_encoding, p);
-  }
-
-  if (!(nativeObj->term = vt_create_term(
-            "xterm", cols, rows, tab_size, 0, encoding, is_auto_encoding, use_auto_detect,
-            logging_vt_seq, unicode_policy, col_size_a, use_char_combining, use_multi_col_char,
-            0 /* use_ctl */, 0 /* bidi_mode */, NULL /* bidi_separators */,
-            0 /* use_dynamic_comb */, BSM_STATIC, 0 /* vertical_mode */, use_local_echo, NULL, NULL,
-            use_ansi_colors, 0 /* alt_color_mode */, 0 /* use_ot_layout */, 0 /* cursor_style */,
-            0 /* ignore_broadcasted_chars */))) {
-    goto error;
+  } else {
+    encoding_str = NULL;
   }
 
   nativeObj->pty_listener.self = nativeObj;
@@ -717,9 +421,6 @@ Java_mlterm_MLTermPty_nativeOpen(JNIEnv *env, jobject obj, jstring jstr_host,
   nativeObj->xterm_listener.set_mouse_report = set_mouse_report;
   nativeObj->xterm_listener.bel = bel;
 
-  vt_term_attach(nativeObj->term, &nativeObj->xterm_listener, &nativeObj->config_listener,
-                 &nativeObj->screen_listener, &nativeObj->pty_listener);
-
   if (jstr_host) {
     host = (*env)->GetStringUTFChars(env, jstr_host, NULL);
   } else if (!(host = getenv("DISPLAY"))) {
@@ -740,39 +441,15 @@ Java_mlterm_MLTermPty_nativeOpen(JNIEnv *env, jobject obj, jstring jstr_host,
     argv = alloca(sizeof(char *) * (len + 1));
 
     for (count = 0; count < len; count++) {
-      argv[count] = (*env)->GetStringUTFChars(
-          env, (*env)->GetObjectArrayElement(env, jarray_argv, count), NULL);
+      char *p;
+      jobject obj = (*env)->GetObjectArrayElement(env, jarray_argv, count);
+      p = (*env)->GetStringUTFChars(env, obj, NULL);
+      argv[count] = bl_str_alloca_dup(p);
+      (*env)->ReleaseStringUTFChars(env, obj, p);
     }
     argv[count] = NULL;
-    cmd_path = argv[0];
-  } else if (default_argv) {
-    argv = default_argv;
-    cmd_path = default_cmd_path;
   } else {
-#ifndef USE_WIN32API
-    if (pass)
-#endif
-    {
-      cmd_path = NULL;
-      argv = alloca(sizeof(char *));
-      argv[0] = NULL;
-    }
-#ifndef USE_WIN32API
-    else {
-      cmd_path = getenv("SHELL");
-      argv = alloca(sizeof(char *) * 2);
-      argv[1] = NULL;
-    }
-#endif
-  }
-
-  if (cmd_path) {
-    if (use_login_shell) {
-      argv[0] = alloca(strlen(cmd_path) + 2);
-      sprintf(argv[0], "-%s", bl_basename(cmd_path));
-    } else {
-      argv[0] = bl_basename(cmd_path);
-    }
+    argv = NULL;
   }
 
   envv[0] = alloca(8 + strlen(host) + 1);
@@ -785,24 +462,14 @@ Java_mlterm_MLTermPty_nativeOpen(JNIEnv *env, jobject obj, jstring jstr_host,
   env_for_dialog = env;
 #endif
 
-  ret = vt_term_open_pty(nativeObj->term, cmd_path, argv, envv, host, NULL, pass, public_key,
-                         private_key, 0, 0);
+  nativeObj->term = mlterm_open(host, pass, cols, rows, 0, encoding_str, argv,
+                                &nativeObj->xterm_listener, &nativeObj->config_listener,
+                                &nativeObj->screen_listener, &nativeObj->pty_listener, 1);
 
 #ifdef USE_LIBSSH2
   env_for_dialog = NULL;
 #endif
 
-  if (jarray_argv) {
-    jsize count;
-
-    argv[0] = cmd_path;
-
-    for (count = 0; argv[count]; count++) {
-      (*env)->ReleaseStringUTFChars(env, (*env)->GetObjectArrayElement(env, jarray_argv, count),
-                                    argv[count]);
-    }
-  }
-
   if (jstr_host) {
     (*env)->ReleaseStringUTFChars(env, jstr_host, host);
   }
@@ -811,11 +478,10 @@ Java_mlterm_MLTermPty_nativeOpen(JNIEnv *env, jobject obj, jstring jstr_host,
     (*env)->ReleaseStringUTFChars(env, jstr_pass, pass);
   }
 
-  if (ret) {
+  if (nativeObj->term) {
     return nativeObj;
   }
 
-error:
 #ifdef __DEBUG
   bl_debug_printf(BL_DEBUG_TAG " Failed to open pty.\n");
 #endif
@@ -873,6 +539,7 @@ JNIEXPORT jboolean JNICALL Java_mlterm_MLTermPty_waitForReading(JNIEnv *env, jcl
   struct timeval tval;
   int *xssh_fds;
   u_int num_xssh_fds;
+  static u_int keepalive_msec_left;
 #endif
 
   vt_close_dead_terms();
@@ -914,11 +581,11 @@ JNIEXPORT jboolean JNICALL Java_mlterm_MLTermPty_waitForReading(JNIEnv *env, jcl
       }
     }
 
+    tval.tv_sec = (keepalive_msec_left + 999) / 1000;
     tval.tv_usec = 0;
-    tval.tv_sec = keepalive_interval;
 
-    if (select(maxfd + 1, &read_fds, NULL, NULL, keepalive_interval > 0 ? &tval : NULL) == 0) {
-      vt_pty_ssh_keepalive(keepalive_interval * 1000);
+    if (select(maxfd + 1, &read_fds, NULL, NULL, tval.tv_sec > 0 ? &tval : NULL) == 0) {
+      keepalive_msec_left = vt_pty_ssh_keepalive(tval.tv_sec * 1000);
     } else
 #else
     select(maxfd + 1, &read_fds, NULL, NULL, NULL);
