@@ -120,16 +120,8 @@ int main(int argc, char **argv)
 {
 #if defined(USE_WIN32API) && defined(USE_LIBSSH2)
   WSADATA wsadata;
-  extern DWORD main_tid; /* see libptyssh/vt_pty_ssh.c */
 
   WSAStartup(MAKEWORD(2, 0), &wsadata);
-
-  /*
-   * XXX
-   * vt_pty_ssh_new() isn't called from the main thread, so main_tid
-   * must be set here, not in vt_pty_ssh_new().
-   */
-  main_tid = GetCurrentThreadId();
 #endif
 
 #ifdef USE_WIN32API
@@ -138,9 +130,12 @@ int main(int argc, char **argv)
 
   check_console();
 
+  /* Don't call before NSApplicationMain() */
+#if !defined(USE_SDL2) || !defined(__APPLE__)
   /* normal user */
   bl_priv_change_euid(bl_getuid());
   bl_priv_change_egid(bl_getgid());
+#endif
 
   bl_set_sys_conf_dir(CONFIG_PATH);
 
