@@ -26,7 +26,8 @@
 #include "version.h"
 #include "daemon.h"
 
-#if defined(__ANDROID__) || defined(USE_QUARTZ) || defined(USE_WIN32API)
+#if defined(__ANDROID__) || defined(USE_QUARTZ) || (defined(USE_SDL2) && defined(__APPLE__)) || \
+  defined(USE_WIN32API)
 /* Shrink unused memory */
 #define bl_conf_add_opt(conf, a, b, c, d, e) bl_conf_add_opt(conf, a, b, c, d, "")
 #endif
@@ -114,7 +115,7 @@ int main_loop_init(int argc, char **argv) {
   u_int depth;
   char *invalid_msg = "%s %s is not valid.\n";
   char *orig_argv;
-#if defined(USE_FRAMEBUFFER) || defined(USE_WAYLAND)
+#if defined(USE_FRAMEBUFFER) || defined(USE_WAYLAND) || defined(USE_SDL2)
   int use_aafont = 0;
 #endif
 
@@ -227,7 +228,7 @@ int main_loop_init(int argc, char **argv) {
 #if defined(USE_FREETYPE) && defined(USE_FONTCONFIG)
   bl_conf_add_opt(conf, '\0', "aafont", 1, "use_aafont",
                   "use [tv]aafont files with the use of fontconfig"
-#ifdef USE_WAYLAND
+#if defined(USE_WAYLAND) || defined(USE_SDL2)
                   " [true]"
 #else /* USE_FRAMEBUFFER */
                   " [false]"
@@ -237,7 +238,7 @@ int main_loop_init(int argc, char **argv) {
 
   orig_argv = argv;
   if (!bl_conf_parse_args(conf, &argc, &argv,
-#ifdef USE_QUARTZ
+#if defined(USE_QUARTZ) || (defined(USE_SDL2) && defined(__APPLE__))
                           1
 #else
                           0
@@ -380,6 +381,7 @@ int main_loop_init(int argc, char **argv) {
     bl_str_to_int(&kbd_repeat_N, value);
   }
 #endif
+
 #ifdef USE_FRAMEBUFFER
 #if defined(__OpenBSD__) || defined(__NetBSD__)
   if ((value = bl_conf_get_value(conf, "fb_resolution"))) {
@@ -391,8 +393,9 @@ int main_loop_init(int argc, char **argv) {
   }
 #endif
 #endif
+
 #if defined(USE_FREETYPE) && defined(USE_FONTCONFIG)
-#ifdef USE_WAYLAND
+#if defined(USE_WAYLAND) || defined(USE_SDL2)
   if (!(value = bl_conf_get_value(conf, "use_aafont")) || strcmp(value, "false") != 0) {
     ui_use_aafont();
     use_aafont = 1;
@@ -408,14 +411,29 @@ int main_loop_init(int argc, char **argv) {
   ui_main_config_init(&main_config, conf, argc, argv);
 
   if ((value = bl_conf_get_value(conf, "default_font"))) {
-#if defined(USE_WAYLAND) || defined(USE_FRAMEBUFFER)
+#if defined(USE_WAYLAND) || defined(USE_FRAMEBUFFER) || defined(USE_SDL2)
     ui_customize_font_file(use_aafont ? "aafont" : "font", "DEFAULT", value, 0);
     ui_customize_font_file(use_aafont ? "aafont" : "font", "ISO10646_UCS4_1", value, 0);
 #else
     ui_customize_font_file(main_config.type_engine == TYPE_XCORE ? "font" : "aafont", "DEFAULT",
                            value, 0);
+#if defined(USE_XLIB)
+    if (main_config.type_engine == TYPE_XCORE) {
+      char *ucs_font;
+
+      if (*(value + strlen(value) - 1) == '-' && ((ucs_font = alloca(strlen(value) + 11)))) {
+        sprintf(ucs_font, "%siso10646-1", value);
+        ui_customize_font_file("font", "ISO10646_UCS4_1", ucs_font, 0);
+      } else {
+        ui_customize_font_file("font", "ISO10646_UCS4_1", value, 0);
+      }
+    } else {
+      ui_customize_font_file("aafont", "ISO10646_UCS4_1", value, 0);
+    }
+#else
     ui_customize_font_file(main_config.type_engine == TYPE_XCORE ? "font" : "aafont",
                            "ISO10646_UCS4_1", value, 0);
+#endif
 #endif
   }
 
