@@ -36,7 +36,7 @@ static ui_color_t black = {TP_COLOR, 0, 0, 0, 0};
 #define LINE_HEIGHT (win->disp->display->line_height)
 
 /* XXX Check if win is input method window or not. */
-#define IS_IM_WINDOW(win) ((win)->disp->num_of_roots >= 2 && (win) == (win)->disp->roots[1])
+#define IS_IM_WINDOW(win) ((win)->disp->num_roots >= 2 && (win) == (win)->disp->roots[1])
 
 /* --- static variables --- */
 
@@ -116,7 +116,7 @@ static int scroll_region(ui_window_t *win, int src_x, int src_y, u_int width, u_
 }
 
 static void set_attr(FILE *fp, vt_font_t font, u_int fg_pixel, u_int bg_pixel,
-                     int underline_style, int size_attr) {
+                     int line_style, int size_attr) {
   static int size_attr_set;
 
   if (fg_pixel < 0x8) {
@@ -135,13 +135,18 @@ static void set_attr(FILE *fp, vt_font_t font, u_int fg_pixel, u_int bg_pixel,
     fprintf(fp, "\x1b[48;5;%dm", bg_pixel);
   }
 
-  switch (underline_style) {
-    case UNDERLINE_NORMAL:
-      fwrite("\x1b[4m", 1, 4, fp);
-      break;
-    case UNDERLINE_DOUBLE:
-      fwrite("\x1b[21m", 1, 5, fp);
-      break;
+  if (line_style & LS_UNDERLINE_SINGLE) {
+    fwrite("\x1b[4m", 1, 4, fp);
+  } else if (line_style & LS_UNDERLINE_DOUBLE) {
+    fwrite("\x1b[21m", 1, 5, fp);
+  }
+
+  if (line_style & LS_CROSSED_OUT) {
+    fwrite("\x1b[9m", 1, 4, fp);
+  }
+
+  if (line_style & LS_OVERLINE) {
+    fwrite("\x1b[53m", 1, 5, fp);
   }
 
   if (font & FONT_BOLD) {
@@ -169,14 +174,14 @@ static void set_attr(FILE *fp, vt_font_t font, u_int fg_pixel, u_int bg_pixel,
   }
 }
 
-static int draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
-                       ui_color_t *bg_color,      /* must be NULL if wall_picture_bg is 1 */
-                       int x, int y, u_char *str, /* 'len * ch_len' bytes */
-                       u_int len, u_int ch_len, int underline_style) {
+static void draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
+                        ui_color_t *bg_color,      /* must be NULL if wall_picture_bg is 1 */
+                        int x, int y, u_char *str, /* 'len * ch_len' bytes */
+                        u_int len, u_int ch_len, int line_style) {
   u_char *str2;
 
   if (!win->is_mapped) {
-    return 0;
+    return;
   }
 
   if ((ch_len != 1 || str[0] >= 0x80) && (str2 = alloca(len * UTF_MAX_SIZE))) {
@@ -214,7 +219,7 @@ static int draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
      * font->x_off is calculated by the right-justified in ui_font.c.
      */
     if (font->x_off >= font->width / 2) {
-      if (str2 = alloca(len * 2)) {
+      if ((str2 = alloca(len * 2))) {
         u_int count;
 
         for (count = 0; count < len; count++) {
@@ -234,13 +239,11 @@ static int draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
           (win->y + win->vmargin + y) / LINE_HEIGHT + 1 +
             (font->size_attr == DOUBLE_HEIGHT_BOTTOM ? 1 : 0),
           (win->x + win->hmargin + x) / COL_WIDTH + 1);
-  set_attr(win->disp->display->fp, font->id, fg_color->pixel, bg_color->pixel, underline_style,
+  set_attr(win->disp->display->fp, font->id, fg_color->pixel, bg_color->pixel, line_style,
            font->size_attr);
   fwrite(str, 1, len, win->disp->display->fp);
   fwrite("\x1b[m", 1, 3, win->disp->display->fp);
   fflush(win->disp->display->fp);
-
-  return 1;
 }
 
 #ifdef USE_LIBSIXEL
@@ -325,7 +328,7 @@ static int copy_area(ui_window_t *win, Pixmap src, int src_x, /* can be minus */
 #define copy_area(win, src, src_x, src_y, width, height, dst_x, dst_y, accept_margin) (0)
 #endif
 
-static int clear_margin_area(ui_window_t *win) {
+static void clear_margin_area(ui_window_t *win) {
   u_int right_margin;
   u_int bottom_margin;
 
@@ -341,7 +344,7 @@ static int clear_margin_area(ui_window_t *win) {
   }
 
   /* XXX */
-  if (win->num_of_children == 2 &&
+  if (win->num_children == 2 &&
       ACTUAL_HEIGHT(win->children[0]) == ACTUAL_HEIGHT(win->children[1])) {
     if (win->children[0]->x + ACTUAL_WIDTH(win->children[0]) <= win->children[1]->x) {
       ui_window_clear(win, win->children[0]->x + ACTUAL_WIDTH(win->children[0]), 0,
@@ -353,8 +356,6 @@ static int clear_margin_area(ui_window_t *win) {
                       win->height);
     }
   }
-
-  return 1;
 }
 
 static int fix_rl_boundary(ui_window_t *win, int boundary_start, int *boundary_end) {
@@ -390,18 +391,18 @@ static void reset_input_focus(ui_window_t *win) {
     }
   }
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     reset_input_focus(win->children[count]);
   }
 }
 
 #if 0
 static int check_child_window_area(ui_window_t *win) {
-  if (win->num_of_children > 0) {
+  if (win->num_children > 0) {
     u_int count;
     u_int sum;
 
-    for (sum = 0, count = 1; count < win->num_of_children; count++) {
+    for (sum = 0, count = 1; count < win->num_children; count++) {
       sum += (ACTUAL_WIDTH(win->children[count]) * ACTUAL_HEIGHT(win->children[count]));
     }
 
@@ -446,10 +447,10 @@ int ui_window_init(ui_window_t *win, u_int width, u_int height, u_int min_width,
   return 1;
 }
 
-int ui_window_final(ui_window_t *win) {
+void ui_window_final(ui_window_t *win) {
   u_int count;
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     ui_window_final(win->children[count]);
   }
 
@@ -460,25 +461,19 @@ int ui_window_final(ui_window_t *win) {
   if (win->window_finalized) {
     (*win->window_finalized)(win);
   }
-
-  return 1;
 }
 
-int ui_window_set_type_engine(ui_window_t *win, ui_type_engine_t type_engine) { return 1; }
+void ui_window_set_type_engine(ui_window_t *win, ui_type_engine_t type_engine) {}
 
-int ui_window_add_event_mask(ui_window_t *win, long event_mask) {
+void ui_window_add_event_mask(ui_window_t *win, long event_mask) {
   win->event_mask |= event_mask;
-
-  return 1;
 }
 
-int ui_window_remove_event_mask(ui_window_t *win, long event_mask) {
+void ui_window_remove_event_mask(ui_window_t *win, long event_mask) {
   win->event_mask &= ~event_mask;
-
-  return 1;
 }
 
-int ui_window_ungrab_pointer(ui_window_t *win) { return 0; }
+void ui_window_ungrab_pointer(ui_window_t *win) {}
 
 int ui_window_set_wall_picture(ui_window_t *win, Pixmap pic, int do_expose) {
   u_int count;
@@ -499,7 +494,7 @@ int ui_window_set_wall_picture(ui_window_t *win, Pixmap pic, int do_expose) {
 #endif
   }
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     ui_window_set_wall_picture(win->children[count], ParentRelative, do_expose);
   }
 
@@ -525,7 +520,7 @@ int ui_window_unset_wall_picture(ui_window_t *win, int do_expose) {
 #endif
   }
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     ui_window_unset_wall_picture(win->children[count], do_expose);
   }
 
@@ -540,19 +535,25 @@ int ui_window_set_transparent(
 
 int ui_window_unset_transparent(ui_window_t *win) { return 0; }
 
-int ui_window_set_cursor(ui_window_t *win, u_int cursor_shape) {
+void ui_window_set_cursor(ui_window_t *win, u_int cursor_shape) {
   win->cursor_shape = cursor_shape;
-
-  return 1;
 }
 
 int ui_window_set_fg_color(ui_window_t *win, ui_color_t *fg_color) {
+  if (win->fg_color.pixel == fg_color->pixel) {
+    return 0;
+  }
+
   win->fg_color = *fg_color;
 
   return 1;
 }
 
 int ui_window_set_bg_color(ui_window_t *win, ui_color_t *bg_color) {
+  if (win->bg_color.pixel == bg_color->pixel) {
+    return 0;
+  }
+
   win->bg_color = *bg_color;
 
   clear_margin_area(win);
@@ -568,7 +569,7 @@ int ui_window_add_child(ui_window_t *win, ui_window_t *child, int x, int y, int
     return 0;
   }
 
-  if ((p = realloc(win->children, sizeof(*win->children) * (win->num_of_children + 1))) == NULL) {
+  if ((p = realloc(win->children, sizeof(*win->children) * (win->num_children + 1))) == NULL) {
 #ifdef DEBUG
     bl_warn_printf(BL_DEBUG_TAG " realloc failed.\n");
 #endif
@@ -591,7 +592,7 @@ int ui_window_add_child(ui_window_t *win, ui_window_t *child, int x, int y, int
     }
   }
 
-  win->children[win->num_of_children++] = child;
+  win->children[win->num_children++] = child;
 
   return 1;
 }
@@ -599,11 +600,11 @@ int ui_window_add_child(ui_window_t *win, ui_window_t *child, int x, int y, int
 int ui_window_remove_child(ui_window_t *win, ui_window_t *child) {
   u_int count;
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     if (win->children[count] == child) {
       child->parent = NULL;
 
-      win->children[count] = win->children[--win->num_of_children];
+      win->children[count] = win->children[--win->num_children];
 
       return 1;
     }
@@ -669,7 +670,7 @@ int ui_window_show(ui_window_t *win,
    * showing child windows.
    */
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     ui_window_show(win->children[count], 0);
   }
 
@@ -680,21 +681,17 @@ int ui_window_show(ui_window_t *win,
   return 1;
 }
 
-int ui_window_map(ui_window_t *win) {
+void ui_window_map(ui_window_t *win) {
   if (!win->is_mapped) {
     win->is_mapped = 1;
 
     (*win->window_exposed)(win, 0, 0, ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win));
     clear_margin_area(win);
   }
-
-  return 1;
 }
 
-int ui_window_unmap(ui_window_t *win) {
+void ui_window_unmap(ui_window_t *win) {
   win->is_mapped = 0;
-
-  return 1;
 }
 
 int ui_window_resize(ui_window_t *win, u_int width, /* excluding margin */
@@ -762,12 +759,12 @@ int ui_window_resize_with_margin(ui_window_t *win, u_int width, u_int height,
   return ui_window_resize(win, width - win->hmargin * 2, height - win->vmargin * 2, flag);
 }
 
-int ui_window_set_normal_hints(ui_window_t *win, u_int min_width, u_int min_height, u_int width_inc,
-                               u_int height_inc) {
-  return 1;
-}
+void ui_window_set_maximize_flag(ui_window_t *win, ui_maximize_flag_t flag) {}
 
-int ui_window_set_override_redirect(ui_window_t *win, int flag) { return 0; }
+void ui_window_set_normal_hints(ui_window_t *win, u_int min_width, u_int min_height,
+                                u_int width_inc, u_int height_inc) {}
+
+void ui_window_set_override_redirect(ui_window_t *win, int flag) {}
 
 int ui_window_set_borderless_flag(ui_window_t *win, int flag) { return 0; }
 
@@ -829,9 +826,9 @@ int ui_window_move(ui_window_t *win, int x, int y) {
   return 1;
 }
 
-int ui_window_clear(ui_window_t *win, int x, int y, u_int width, u_int height) {
+void ui_window_clear(ui_window_t *win, int x, int y, u_int width, u_int height) {
   if (!win->wall_picture) {
-    return ui_window_fill_with(win, &win->bg_color, x, y, width, height);
+    ui_window_fill_with(win, &win->bg_color, x, y, width, height);
   } else {
     Pixmap pic;
     int src_x;
@@ -847,31 +844,29 @@ int ui_window_clear(ui_window_t *win, int x, int y, u_int width, u_int height) {
       src_y = y;
     }
 
-    return copy_area(win, pic, src_x, src_y, width, height, x, y, 1);
+    copy_area(win, pic, src_x, src_y, width, height, x, y, 1);
   }
-
-  return 1;
 }
 
-int ui_window_clear_all(ui_window_t *win) {
-  return ui_window_clear(win, 0, 0, win->width, win->height);
+void ui_window_clear_all(ui_window_t *win) {
+  ui_window_clear(win, 0, 0, win->width, win->height);
 }
 
-int ui_window_fill(ui_window_t *win, int x, int y, u_int width, u_int height) {
-  return ui_window_fill_with(win, &win->fg_color, x, y, width, height);
+void ui_window_fill(ui_window_t *win, int x, int y, u_int width, u_int height) {
+  ui_window_fill_with(win, &win->fg_color, x, y, width, height);
 }
 
-int ui_window_fill_with(ui_window_t *win, ui_color_t *color, int x, int y, u_int width,
-                        u_int height) {
+void ui_window_fill_with(ui_window_t *win, ui_color_t *color, int x, int y, u_int width,
+                         u_int height) {
   u_int h;
   int fill_to_end;
 
   if (!win->is_mapped) {
-    return 0;
+    return;
   }
 
   if (height < LINE_HEIGHT || width < COL_WIDTH) {
-    return 0;
+    return;
   }
 
   if (!IS_IM_WINDOW(win) && (!win->parent || win->x + ACTUAL_WIDTH(win) >= win->parent->width) &&
@@ -913,32 +908,28 @@ int ui_window_fill_with(ui_window_t *win, ui_color_t *color, int x, int y, u_int
 
   fwrite("\x1b[m", 1, 3, win->disp->display->fp);
   fflush(win->disp->display->fp);
-
-  return 1;
 }
 
-int ui_window_blank(ui_window_t *win) {
-  return ui_window_fill_with(win, &win->fg_color, 0, 0, win->width - RIGHT_MARGIN(win),
-                             win->height - BOTTOM_MARGIN(win));
+void ui_window_blank(ui_window_t *win) {
+  ui_window_fill_with(win, &win->fg_color, 0, 0, win->width - RIGHT_MARGIN(win),
+                      win->height - BOTTOM_MARGIN(win));
 }
 
-int ui_window_update(ui_window_t *win, int flag) {
+void ui_window_update(ui_window_t *win, int flag) {
   if (!win->is_mapped) {
-    return 0;
+    return;
   }
 
   if (win->update_window) {
     (*win->update_window)(win, flag);
   }
-
-  return 1;
 }
 
-int ui_window_update_all(ui_window_t *win) {
+void ui_window_update_all(ui_window_t *win) {
   u_int count;
 
   if (!win->is_mapped) {
-    return 0;
+    return;
   }
 
   if (!win->parent) {
@@ -951,17 +942,15 @@ int ui_window_update_all(ui_window_t *win) {
     (*win->window_exposed)(win, 0, 0, win->width, win->height);
   }
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     ui_window_update_all(win->children[count]);
   }
-
-  return 1;
 }
 
 void ui_window_idling(ui_window_t *win) {
   u_int count;
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     ui_window_idling(win->children[count]);
   }
 
@@ -986,7 +975,7 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
 #if 0
   u_int count;
 
-  for (count = 0; count < win->num_of_children; count++) {
+  for (count = 0; count < win->num_children; count++) {
     if (ui_window_receive_event(win->children[count], event)) {
       return 1;
     }
@@ -1285,49 +1274,45 @@ void ui_window_set_clip(ui_window_t *win, int x, int y, u_int width, u_int heigh
 
 void ui_window_unset_clip(ui_window_t *win) {}
 
-int ui_window_console_draw_decsp_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
-                                        ui_color_t *bg_color, int x, int y, u_char *str, u_int len,
-                                        int underline_style) {
+void ui_window_console_draw_decsp_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
+                                         ui_color_t *bg_color, int x, int y, u_char *str, u_int len,
+                                         int line_style) {
   if (!win->is_mapped) {
-    return 0;
+    return;
   }
 
   fprintf(win->disp->display->fp, "\x1b[%d;%dH",
           (win->y + win->vmargin + y) / LINE_HEIGHT + 1 +
             (font->size_attr == DOUBLE_HEIGHT_BOTTOM ? 1 : 0),
           (win->x + win->hmargin + x) / COL_WIDTH + 1);
-  set_attr(win->disp->display->fp, font->id, fg_color->pixel, bg_color->pixel, underline_style,
+  set_attr(win->disp->display->fp, font->id, fg_color->pixel, bg_color->pixel, line_style,
            font->size_attr);
   fwrite("\x1b(0", 1, 3, win->disp->display->fp);
   fwrite(str, 1, len, win->disp->display->fp);
   fwrite("\x1b(B\x1b[m", 1, 6, win->disp->display->fp);
   fflush(win->disp->display->fp);
-
-  return 1;
 }
 
-int ui_window_console_draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
-                                  ui_color_t *bg_color, int x, int y, u_char *str, u_int len,
-                                  int underline_style) {
-  return draw_string(win, font, fg_color, bg_color, x, y, str, len, 1, underline_style);
+void ui_window_console_draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
+                                   ui_color_t *bg_color, int x, int y, u_char *str, u_int len,
+                                   int line_style) {
+  draw_string(win, font, fg_color, bg_color, x, y, str, len, 1, line_style);
 }
 
-int ui_window_console_draw_string16(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
-                                    ui_color_t *bg_color, int x, int y, XChar2b *str, u_int len,
-                                    int underline_style) {
-  return draw_string(win, font, fg_color, bg_color, x, y, str, len, 2, underline_style);
+void ui_window_console_draw_string16(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color,
+                                     ui_color_t *bg_color, int x, int y, XChar2b *str, u_int len,
+                                     int line_style) {
+  draw_string(win, font, fg_color, bg_color, x, y, str, len, 2, line_style);
 }
 
-int ui_window_draw_rect_frame(ui_window_t *win, int x1, int y1, int x2, int y2) {
+void ui_window_draw_rect_frame(ui_window_t *win, int x1, int y1, int x2, int y2) {
   ui_window_fill_with(win, &win->fg_color, x1, y1, x2 - x1 + 1, 1);
   ui_window_fill_with(win, &win->fg_color, x1, y1, 1, y2 - y1 + 1);
   ui_window_fill_with(win, &win->fg_color, x1, y2, x2 - x1 + 1, 1);
   ui_window_fill_with(win, &win->fg_color, x2, y1, 1, y2 - y1 + 1);
-
-  return 1;
 }
 
-int ui_set_use_clipboard_selection(int use_it) { return 0; }
+void ui_set_use_clipboard_selection(int use_it) {}
 
 int ui_is_using_clipboard_selection(void) { return 0; }
 
@@ -1343,10 +1328,10 @@ int ui_window_set_selection_owner(ui_window_t *win, Time time) {
 
 int ui_window_xct_selection_request(ui_window_t *win, Time time) {
   u_int count;
-  u_int num_of_displays;
-  ui_display_t **displays = ui_get_opened_displays(&num_of_displays);
+  u_int num_displays;
+  ui_display_t **displays = ui_get_opened_displays(&num_displays);
 
-  for (count = 0; count < num_of_displays; count++) {
+  for (count = 0; count < num_displays; count++) {
     if (displays[count]->selection_owner) {
       ui_window_t *owner = displays[count]->selection_owner;
       if (owner->xct_selection_requested) {
@@ -1365,10 +1350,10 @@ int ui_window_xct_selection_request(ui_window_t *win, Time time) {
 
 int ui_window_utf_selection_request(ui_window_t *win, Time time) {
   u_int count;
-  u_int num_of_displays;
-  ui_display_t **displays = ui_get_opened_displays(&num_of_displays);
+  u_int num_displays;
+  ui_display_t **displays = ui_get_opened_displays(&num_displays);
 
-  for (count = 0; count < num_of_displays; count++) {
+  for (count = 0; count < num_displays; count++) {
     if (displays[count]->selection_owner) {
       ui_window_t *owner = displays[count]->selection_owner;
       if (owner->utf_selection_requested) {
@@ -1385,12 +1370,10 @@ int ui_window_utf_selection_request(ui_window_t *win, Time time) {
   return 1;
 }
 
-int ui_window_send_picture_selection(ui_window_t *win, Pixmap pixmap, u_int width, u_int height) {
-  return 0;
-}
+void ui_window_send_picture_selection(ui_window_t *win, Pixmap pixmap, u_int width, u_int height) {}
 
-int ui_window_send_text_selection(ui_window_t *win, XSelectionRequestEvent *req_ev,
-                                  u_char *sel_data, size_t sel_len, Atom sel_type) {
+void ui_window_send_text_selection(ui_window_t *win, XSelectionRequestEvent *req_ev,
+                                   u_char *sel_data, size_t sel_len, Atom sel_type) {
   if (req_ev) {
     if (req_ev->type == 1) {
       if (req_ev->target->utf_selection_notified) {
@@ -1402,11 +1385,9 @@ int ui_window_send_text_selection(ui_window_t *win, XSelectionRequestEvent *req_
       }
     }
   }
-
-  return 1;
 }
 
-int ui_set_window_name(ui_window_t *win, u_char *name) {
+void ui_set_window_name(ui_window_t *win, u_char *name) {
   vt_char_encoding_t encoding;
   ef_parser_t *parser;
 
@@ -1435,14 +1416,10 @@ int ui_set_window_name(ui_window_t *win, u_char *name) {
     fwrite("\x07", 1, 1, win->disp->display->fp);
 
     (*parser->delete)(parser);
-  } else {
-    return 0;
   }
-
-  return 1;
 }
 
-int ui_set_icon_name(ui_window_t *win, u_char *name) {
+void ui_set_icon_name(ui_window_t *win, u_char *name) {
   vt_char_encoding_t encoding;
   ef_parser_t *parser;
 
@@ -1471,50 +1448,41 @@ int ui_set_icon_name(ui_window_t *win, u_char *name) {
 
     (*parser->delete)(parser);
   }
-
-  return 1;
 }
 
-int ui_window_set_icon(ui_window_t *win, ui_icon_picture_ptr_t icon) { return 1; }
+void ui_window_set_icon(ui_window_t *win, ui_icon_picture_ptr_t icon) {}
 
-int ui_window_remove_icon(ui_window_t *win) { return 1; }
+void ui_window_remove_icon(ui_window_t *win) {}
 
-int ui_window_reset_group(ui_window_t *win) { return 1; }
+void ui_window_reset_group(ui_window_t *win) {}
 
-int ui_window_get_visible_geometry(ui_window_t *win, int *x, /* x relative to root window */
-                                   int *y,                   /* y relative to root window */
-                                   int *my_x,                /* x relative to my window */
-                                   int *my_y,                /* y relative to my window */
-                                   u_int *width, u_int *height) {
-  return 1;
-}
-
-int ui_set_click_interval(int interval) {
+void ui_set_click_interval(int interval) {
   click_interval = interval;
+}
 
-  return 1;
+int ui_get_click_interval(void) {
+  return click_interval;
 }
 
 u_int ui_window_get_mod_ignore_mask(ui_window_t *win, KeySym *keysyms) { return ~0; }
 
 u_int ui_window_get_mod_meta_mask(ui_window_t *win, char *mod_key) { return ModMask; }
 
-int ui_window_bell(ui_window_t *win, ui_bel_mode_t mode) {
+void ui_window_bell(ui_window_t *win, ui_bel_mode_t mode) {
   if (mode & BEL_VISUAL) {
     ui_window_blank(win);
     bl_usleep(100000); /* 100 mili sec */
     (*win->window_exposed)(win, 0, 0, win->width, win->height);
   }
 
-  return 1;
+  if (mode & BEL_SOUND) {
+    fwrite("\x07", 1, 1, win->disp->display->fp);
+  }
 }
 
-int ui_window_translate_coordinates(ui_window_t *win, int x, int y, int *global_x, int *global_y,
-                                    Window *child) {
+void ui_window_translate_coordinates(ui_window_t *win, int x, int y, int *global_x, int *global_y) {
   *global_x = x + win->x;
   *global_y = y + win->y;
-
-  return 1;
 }
 
 void ui_window_set_input_focus(ui_window_t *win) {
@@ -1527,4 +1495,4 @@ void ui_window_set_input_focus(ui_window_t *win) {
 
 /* for ui_display.c */
 
-int ui_window_clear_margin_area(ui_window_t *win) { return clear_margin_area(win); }
+void ui_window_clear_margin_area(ui_window_t *win) { clear_margin_area(win); }
