@@ -6,9 +6,9 @@
 
 /* --- global functions --- */
 
-int ui_event_source_init(void) { return 1; }
+void ui_event_source_init(void) {}
 
-int ui_event_source_final(void) { return 1; }
+void ui_event_source_final(void) {}
 
 int ui_event_source_process(void) { return 1; }
 
@@ -18,8 +18,6 @@ int ui_event_source_add_fd(int fd, void (*handler)(void)) {
   return 1;
 }
 
-int ui_event_source_remove_fd(int fd) {
+void ui_event_source_remove_fd(int fd) {
   cocoa_remove_fd(fd);
-
-  return 1;
 }
