@@ -35,7 +35,7 @@ typedef struct custom_cache {
 
 /* --- static variables --- */
 
-#if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE) || defined(USE_WAYLAND)
+#if defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE) || defined(USE_WAYLAND) || defined(USE_SDL2)
 
 #if defined(USE_FREETYPE) && defined(USE_FONTCONFIG)
 static int use_aafont;
@@ -1176,8 +1176,9 @@ char *ui_font_config_dump(ui_font_config_t *font_config) {
 
   list_len = 0;
   for (count = 0; count < size; count++) {
-    list_len += (4 /* CSI2J */ + 15 /* ISO10646_UCS4_1, U+10FFFF-10FFFF */ + 12 /* _BOLD_ITALIC */ +
-                 1 /* = */ + strlen(array[count]->value) + 2 /* \r\n or \0 */);
+    list_len += (4 /* CSI2J */ + 15 /* ISO10646_UCS4_1, U+10FFFF-10FFFF */ +
+                 12 /* _BOLD_ITALIC */ + 10 /* _FULLWIDTH */ + 1 /* = */ +
+                 strlen(array[count]->value) + 2 /* \r\n or \0 */);
   }
 
   if ((font_name_list = malloc(list_len)) == NULL) {
@@ -1224,6 +1225,11 @@ char *ui_font_config_dump(ui_font_config_t *font_config) {
       p += 7;
     }
 
+    if (array[count]->key & FONT_FULLWIDTH) {
+      strcpy(p, "_FULLWIDTH");
+      p += 10;
+    }
+
     sprintf(p, "=%s", array[count]->value);
     p += strlen(p);
 
