@@ -7,9 +7,11 @@
 #include <pobl/bl_str.h>    /* bl_str_to_uint */
 #include <pobl/bl_locale.h> /* bl_get_lang */
 #include <pobl/bl_args.h>
+#include <pobl/bl_util.h>   /* BL_INT_TO_STR */
 
 #include <vt_term_manager.h>
 #include "ui_selection_encoding.h"
+#include "ui_emoji.h"
 
 #if defined(__ANDROID__) || defined(USE_QUARTZ) || defined(USE_WIN32API)
 /* Shrink unused memory */
@@ -160,7 +162,8 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
   bl_conf_add_opt(conf, 'k', "meta", 0, "mod_meta_mode",
                   "mode in pressing meta key (none/esc/8bit) [8bit]");
   bl_conf_add_opt(conf, 'l', "sl", 0, "logsize",
-                  "number of backlog (scrolled lines to save) [128]");
+                  "number of backlog (scrolled lines to save) ["
+                  BL_INT_TO_STR(VT_LOG_SIZE_UNIT) "]");
   bl_conf_add_opt(conf, 'm', "comb", 1, "use_combining", "use combining characters [true]");
   bl_conf_add_opt(conf, 'n', "noucsfont", 1, "not_use_unicode_font",
                   "use non-Unicode fonts even in UTF-8 mode [false]");
@@ -196,11 +199,13 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
   bl_conf_add_opt(conf, '\0', "parent", 0, "parent_window", "parent window");
   bl_conf_add_opt(conf, '\0', "bd", 0, "bd_color",
                   "Color to use to display bold characters (equivalent to colorBD)");
-  bl_conf_add_opt(conf, '\0', "it", 0, "it_color", "Color to use to display italic characters");
   bl_conf_add_opt(conf, '\0', "ul", 0, "ul_color",
                   "Color to use to display underlined characters (equivalent to colorUL)");
   bl_conf_add_opt(conf, '\0', "bl", 0, "bl_color",
                   "Color to use to display blinking characters (equivalent to colorBL)");
+  bl_conf_add_opt(conf, '\0', "rv", 0, "rv_color",
+                  "Color to use to display reverse characters (equivalent to colorRV)");
+  bl_conf_add_opt(conf, '\0', "it", 0, "it_color", "Color to use to display italic characters");
   bl_conf_add_opt(conf, '\0', "co", 0, "co_color",
                   "Color to use to display crossed-out characters");
   bl_conf_add_opt(conf, '\0', "noul", 1, "hide_underline", "Don't draw underline [false]");
@@ -213,7 +218,6 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
                   "Features of glyph subsutitution [liga,clig,dlig,hlig,rlig]");
 #endif
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
-  bl_conf_add_opt(conf, '\0', "servlist", 0, "server_list", "list of servers to connect");
   bl_conf_add_opt(conf, '\0', "serv", 0, "default_server", "connecting server by default");
   bl_conf_add_opt(conf, '\0', "dialog", 1, "always_show_dialog",
                   "always show dialog to input server address, password and so on [false]");
@@ -270,6 +274,8 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
                   "use unicode fonts partially regardless of -n option");
   bl_conf_add_opt(conf, '\0', "fullwidth", 0, "unicode_full_width_areas",
                   "force full width regardless of EastAsianWidth.txt");
+  bl_conf_add_opt(conf, '\0', "halfwidth", 0, "unicode_half_width_areas",
+                  "force half width regardless of EastAsianWidth.txt");
   bl_conf_add_opt(conf, '\0', "ade", 0, "auto_detect_encodings",
                   "encodings detected automatically");
   bl_conf_add_opt(conf, '\0', "auto", 1, "use_auto_detect",
@@ -318,6 +324,8 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
                   "broadcast input characters to all ptys [false]");
   bl_conf_add_opt(conf, '\0', "ibc", 1, "ignore_broadcasted_chars",
                   "ignore broadcasted characters [false]");
+  bl_conf_add_opt(conf, '\0', "emoji", 0, "emoji_path",
+                  "emoji directory or file path [~/.mlterm/emoji]");
 #ifdef USE_IM_CURSOR_COLOR
   bl_conf_add_opt(conf, '\0', "imcolor", 0, "im_cursor_color",
                   "cursor color when input method is activated. [false]");
@@ -511,11 +519,6 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     main_config->alt_color_mode |= ALT_COLOR_BOLD;
   }
 
-  if ((value = bl_conf_get_value(conf, "it_color"))) {
-    main_config->it_color = strdup(value);
-    main_config->alt_color_mode |= ALT_COLOR_ITALIC;
-  }
-
   if ((value = bl_conf_get_value(conf, "ul_color"))) {
     main_config->ul_color = strdup(value);
     main_config->alt_color_mode |= ALT_COLOR_UNDERLINE;
@@ -526,6 +529,16 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     main_config->alt_color_mode |= ALT_COLOR_BLINKING;
   }
 
+  if ((value = bl_conf_get_value(conf, "rv_color"))) {
+    main_config->rv_color = strdup(value);
+    main_config->alt_color_mode |= ALT_COLOR_REVERSE;
+  }
+
+  if ((value = bl_conf_get_value(conf, "it_color"))) {
+    main_config->it_color = strdup(value);
+    main_config->alt_color_mode |= ALT_COLOR_ITALIC;
+  }
+
   if ((value = bl_conf_get_value(conf, "co_color"))) {
     main_config->ul_color = strdup(value);
     main_config->alt_color_mode |= ALT_COLOR_CROSSED_OUT;
@@ -625,7 +638,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     }
   }
 
-  main_config->num_log_lines = 128;
+  main_config->num_log_lines = VT_LOG_SIZE_UNIT;
   main_config->unlimit_log_size = 0;
 
   if ((value = bl_conf_get_value(conf, "logsize"))) {
@@ -652,7 +665,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     }
   }
 
-#ifdef USE_QUARTZ
+#ifdef __APPLE__
   main_config->use_login_shell = 1;
 #else
   main_config->use_login_shell = 0;
@@ -716,6 +729,10 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     vt_set_full_width_areas(value);
   }
 
+  if ((value = bl_conf_get_value(conf, "unicode_half_width_areas"))) {
+    vt_set_half_width_areas(value);
+  }
+
   if ((value = bl_conf_get_value(conf, "only_use_unicode_font"))) {
     if (strcmp(value, "true") == 0) {
       if (main_config->unicode_policy == NOT_USE_UNICODE_FONT) {
@@ -1033,32 +1050,8 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
   }
 
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
-  if ((value = bl_conf_get_value(conf, "server_list"))) {
-    if ((main_config->server_list = malloc(sizeof(char *) *
-                                           /* A,B => A B NULL */
-                                           (bl_count_char_in_str(value, ',') + 2)))) {
-      if ((value = strdup(value))) {
-        int count;
-
-        count = 0;
-        do {
-          main_config->server_list[count++] = bl_str_sep(&value, ",");
-        } while (value);
-
-        main_config->server_list[count] = NULL;
-      } else {
-        free(main_config->server_list);
-        main_config->server_list = NULL;
-      }
-    }
-  }
-
-  if ((value = bl_conf_get_value(conf, "default_server"))) {
-    if (*value && (main_config->default_server = strdup(value))) {
-#ifdef USE_WIN32API
-      ui_main_config_add_to_server_list(main_config, main_config->default_server);
-#endif
-    }
+  if ((value = bl_conf_get_value(conf, "default_server")) && *value) {
+    main_config->default_server = strdup(value);
   }
 
   if ((value = bl_conf_get_value(conf, "always_show_dialog"))) {
@@ -1325,6 +1318,10 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     }
   }
 
+  if ((value = bl_conf_get_value(conf, "emoji_path"))) {
+    ui_emoji_set_path(value);
+  }
+
 #ifdef USE_IM_CURSOR_COLOR
   if ((value = bl_conf_get_value(conf, "im_cursor_color"))) {
     if (*value) {
@@ -1393,11 +1390,6 @@ void ui_main_config_final(ui_main_config_t *main_config) {
   free(main_config->bidi_separators);
 
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
-  if (main_config->server_list) {
-    free(main_config->server_list[0]);
-    free(main_config->server_list);
-  }
-
   free(main_config->default_server);
 #endif
 
@@ -1409,53 +1401,3 @@ void ui_main_config_final(ui_main_config_t *main_config) {
   free(main_config->work_dir);
   free(main_config->cmd_argv);
 }
-
-#if defined(USE_WIN32API) || defined(USE_LIBSSH2)
-int ui_main_config_add_to_server_list(ui_main_config_t *main_config, char *server) {
-  u_int nlist;
-  size_t len;
-  size_t add_len;
-  char *p;
-  char **pp;
-
-  nlist = 0;
-  len = 0;
-  if (main_config->server_list) {
-    while (main_config->server_list[nlist]) {
-      len += (strlen(main_config->server_list[nlist]) + 1);
-
-      if (strcmp(main_config->server_list[nlist++], server) == 0) {
-        return 1;
-      }
-    }
-  }
-
-  add_len = strlen(server) + 1;
-  p = NULL;
-  if (main_config->server_list) {
-    if ((p = realloc(main_config->server_list[0], len + add_len))) {
-      memcpy(p + len, server, add_len);
-    }
-  } else {
-    p = strdup(server);
-  }
-
-  if (p && (pp = realloc(main_config->server_list, sizeof(char *) * (nlist + 2)))) {
-    u_int count;
-
-    main_config->server_list = pp;
-
-    for (count = 0; count < nlist + 1; count++) {
-      main_config->server_list[count] = p;
-      p += (strlen(p) + 1);
-    }
-    main_config->server_list[count] = NULL;
-
-    return 1;
-  } else {
-    free(p);
-
-    return 0;
-  }
-}
-#endif
