@@ -12,11 +12,20 @@
 #include <pobl/bl_debug.h>
 #include <pobl/bl_str.h>     /* strdup */
 #include <pobl/bl_mem.h>     /* malloc */
-#include <pobl/bl_util.h>    /* K_MIN */
+#include <pobl/bl_util.h>    /* DIGIT_STR_LEN */
 #include <pobl/bl_conf_io.h> /* bl_get_user_rc_path */
 
-#ifdef HAVE_PTHREAD
+#ifdef HAVE_WINDOWS_H
+#include <windows.h> /* for CreateEvent */
+#endif
+
+#if defined(USE_WIN32API)
+#include <process.h> /* _beginthreadex */
+#elif defined(HAVE_PTHREAD)
 #include <pthread.h>
+#endif
+
+#if defined(HAVE_PTHREAD) || defined(USE_WIN32API)
 #ifndef HAVE_WINDOWS_H
 #include <sys/select.h>
 #include <unistd.h>
@@ -44,6 +53,7 @@ typedef struct inline_pic_args {
 #else
   int ev;
 #endif
+  int keep_aspect;
 
 } inline_pic_args_t;
 
@@ -166,7 +176,7 @@ static ui_icon_picture_t *create_icon_picture(ui_display_t *disp,
   }
 
   if (!ui_imagelib_load_file(disp, file_path, &(pic->cardinal), &(pic->pixmap), &(pic->mask),
-                             &icon_size, &icon_size)) {
+                             &icon_size, &icon_size, 0)) {
     free(pic->file_path);
     free(pic);
 
@@ -346,6 +356,22 @@ static void check_inline_pictures(vt_term_t *term, u_int8_t *flags, int beg, int
   }
 }
 
+static void check_inline_pictures_drcs(vt_term_t *term, u_int8_t *flags) {
+  vt_drcs_t *drcs;
+
+  if ((drcs = term->parser->drcs)) {
+    size_t count;
+
+    for (count = 0; count < sizeof(drcs->fonts) / sizeof(drcs->fonts[0]); count++) {
+      if (drcs->fonts[count] && vt_drcs_has_picture(drcs->fonts[count])) {
+        flags[drcs->fonts[count]->pic_id] = 1;
+
+        /* animation check is not necessary. */
+      }
+    }
+  }
+}
+
 static int cleanup_inline_pictures(vt_term_t *term) {
 #define THRESHOLD 48
   int count;
@@ -403,6 +429,8 @@ static int cleanup_inline_pictures(vt_term_t *term) {
     }
 
     term->screen->edit = orig_edit;
+
+    check_inline_pictures_drcs(term, flags);
   }
 
   empty_idx = -1;
@@ -479,7 +507,7 @@ static int load_file(void *p) {
   height = inline_pics[idx].height;
 
   if (ui_imagelib_load_file(inline_pics[idx].disp, inline_pics[idx].file_path, NULL, &pixmap, &mask,
-                            &width, &height)) {
+                            &width, &height, ((inline_pic_args_t*)p)->keep_aspect)) {
     if (strstr(inline_pics[idx].file_path, "mlterm/anim")) {
       /* GIF Animation frame */
       unlink(inline_pics[idx].file_path);
@@ -803,9 +831,13 @@ void ui_release_icon_picture(ui_icon_picture_t *pic) {
   }
 }
 
-int ui_load_inline_picture(ui_display_t *disp, char *file_path, u_int *width, /* can be 0 */
-                           u_int *height,                                     /* can be 0 */
-                           u_int col_width, u_int line_height, vt_term_t *term) {
+/*
+ * The caller must check if file_path exists or not to avoid to call mlimgloader unnecessarily.
+ * 'file_path' can be /foo/bar.ttf/1f400 for emoji.
+ */
+int ui_load_inline_picture(ui_display_t *disp, char *file_path, u_int *width /* can be 0 */,
+                           u_int *height /* can be 0 */, u_int col_width, u_int line_height,
+                           int keep_aspect, vt_term_t *term) {
   int idx;
   inline_pic_args_t *args;
 
@@ -841,6 +873,7 @@ int ui_load_inline_picture(ui_display_t *disp, char *file_path, u_int *width, /*
   }
 
   args->idx = idx;
+  args->keep_aspect = keep_aspect;
 
 #if defined(HAVE_PTHREAD) || defined(USE_WIN32API)
   if (strstr(file_path, "://")) {
@@ -903,20 +936,7 @@ int ui_load_inline_picture(ui_display_t *disp, char *file_path, u_int *width, /*
   } else
 #endif
   {
-    int ret;
-
-#ifdef BUILTIN_IMAGELIB
-    ret = load_file(args);
-#else
-    struct stat st;
-
-    ret = (
-#if !defined(HAVE_PTHREAD) && !defined(USE_WIN32API)
-              strstr(file_path, "://") ||
-#endif
-              stat(file_path, &st) == 0) &&
-          load_file(args);
-#endif
+    int ret = load_file(args);
 
     free(args);
 
@@ -1087,7 +1107,7 @@ int ui_load_tmp_picture(ui_display_t *disp, char *file_path, Pixmap *pixmap, Pix
                         u_int *width, u_int *height) {
   *width = *height = 0;
 
-  if (ui_imagelib_load_file(disp, file_path, NULL, pixmap, mask, width, height)) {
+  if (ui_imagelib_load_file(disp, file_path, NULL, pixmap, mask, width, height, 0)) {
     return 1;
   } else {
     return 0;
