@@ -22,6 +22,7 @@
 #include <pobl/bl_types.h> /* u_int */
 #include <pobl/bl_args.h>  /* bl_arg_str_to_array */
 #include <pobl/bl_sig_child.h>
+#include <pobl/bl_dialog.h>
 #include <vt_term_manager.h>
 #include <vt_char_encoding.h>
 
@@ -148,6 +149,9 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
   char *term_env;
   char *uri;
   char *pass;
+#ifdef USE_LIBSSH2
+  char *privkey;
+#endif
   int ret;
 
   display = win->disp->name;
@@ -196,14 +200,17 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
 
   uri = NULL;
   pass = NULL;
+#ifdef USE_LIBSSH2
+  privkey = main_config.private_key;
+#endif
 
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
   if (show_dialog || main_config.default_server) {
     char *user;
     char *host;
     char *port;
-    char *encoding;
-    char *exec_cmd;
+    char *encoding = NULL;
+    char *exec_cmd = NULL;
     int x11_fwd;
     void *session;
 
@@ -216,24 +223,20 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
         (session = vt_search_ssh_session(host, port, user))) {
       uri = strdup(main_config.default_server);
       pass = strdup("");
-      exec_cmd = NULL;
 
       if (x11_fwd) {
         vt_pty_ssh_set_use_x11_forwarding(session, x11_fwd);
       }
     } else
 #endif
-    if (!ui_connect_dialog(&uri, &pass, &exec_cmd, &x11_fwd, display, window,
-                           main_config.server_list, main_config.default_server)) {
+    if (!ui_connect_dialog(&uri, &pass, &exec_cmd, &privkey, &x11_fwd, display, window,
+                           main_config.default_server)) {
       bl_msg_printf("Connect dialog is canceled.\n");
       if (vt_get_all_terms(NULL) > 1) {
         return 0;
       }
-      encoding = exec_cmd = NULL;
     } else {
-      if (!bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding, bl_str_alloca_dup(uri))) {
-        encoding = NULL;
-      }
+      bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding, bl_str_alloca_dup(uri));
 
 #ifdef USE_LIBSSH2
       vt_pty_ssh_set_use_x11_forwarding(vt_search_ssh_session(host, port, user), x11_fwd);
@@ -297,7 +300,7 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
    * cmd_path is not changed.
    */
   if (!uri && !cmd_path) {
-#ifdef USE_QUARTZ
+#ifdef __APPLE__
     char *user;
 
     if ((user = getenv("USER")) && (cmd_argv = alloca(sizeof(char *) * 4))) {
@@ -353,7 +356,7 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
   ret = vt_term_open_pty(term, cmd_path, cmd_argv, env, uri ? uri : display, main_config.work_dir,
                          pass,
 #ifdef USE_LIBSSH2
-                         main_config.public_key, main_config.private_key,
+                         main_config.public_key, privkey,
 #else
                          NULL, NULL,
 #endif
@@ -362,7 +365,6 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
   if (uri) {
     if (ret && bl_compare_str(uri, main_config.default_server) != 0) {
-      ui_main_config_add_to_server_list(&main_config, uri);
       free(main_config.default_server);
       main_config.default_server = uri;
     } else {
@@ -370,6 +372,10 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
     }
 
     free(pass);
+
+    if (privkey != main_config.private_key) {
+      free(privkey);
+    }
   }
 #endif
 
@@ -378,9 +384,11 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
 
 #ifndef NO_IMAGE
 
-static vt_char_t *get_picture_data(void *p, char *file_path, int *num_cols, /* can be 0 */
-                                   int *num_rows,                           /* can be 0 */
-                                   u_int32_t **sixel_palette) {
+static vt_char_t *get_picture_data(void *p, char *file_path, int *num_cols /* can be 0 */,
+                                   int *num_rows /* can be 0 */,
+                                   int *num_cols_small /* set only if drcs_sixel is 1. */,
+                                   int *num_rows_small /* set only if drcs_sixel is 1. */,
+                                   u_int32_t **sixel_palette, int keep_aspect, int drcs_sixel) {
   vt_char_t *data;
 
   if (num_screens > 0) {
@@ -388,8 +396,10 @@ static vt_char_t *get_picture_data(void *p, char *file_path, int *num_cols, /* c
 
     orig_term = screens[0]->term;
     screens[0]->term = p; /* XXX */
-    data = (*screens[0]->xterm_listener.get_picture_data)(
-        screens[0]->xterm_listener.self, file_path, num_cols, num_rows, sixel_palette);
+    data = (*screens[0]->xterm_listener.get_picture_data)(screens[0]->xterm_listener.self,
+                                                          file_path, num_cols, num_rows,
+                                                          num_cols_small, num_rows_small,
+                                                          sixel_palette, 0, drcs_sixel);
     screens[0]->term = orig_term;
   } else {
     data = NULL;
@@ -528,26 +538,27 @@ static ui_screen_t *open_screen_intern(char *disp_name, vt_term_t *term, ui_layo
                                       usascii_font_cs, main_config.step_in_changing_font_size,
                                       main_config.letter_space, main_config.use_bold_font,
                                       main_config.use_italic_font)) == NULL) {
+    const char *msg_fmt = "No fonts for %s";
+    char *msg;
     char *name;
 
-    name = ui_get_charset_name(usascii_font_cs);
+    if (!(name = ui_get_charset_name(usascii_font_cs))) {
+      name = "US-ASCII";
+    }
 
-    bl_msg_printf(
-        "No fonts found for %s. Please install fonts "
-        "and edit the font config file in ~/.mlterm.\n",
-        name ? name : "US-ASCII");
-
-#ifdef DEBUG
-    bl_warn_printf(BL_DEBUG_TAG " ui_font_manager_new() failed.\n");
-#endif
+    if ((msg = alloca(13 + strlen(name) + 1))) {
+      sprintf(msg, msg_fmt, name);
+      bl_dialog(BL_DIALOG_ALERT, msg);
+    }
 
     goto error;
   }
 
   if ((color_man = ui_color_manager_new(
            disp, main_config.fg_color, main_config.bg_color, main_config.cursor_fg_color,
-           main_config.cursor_bg_color, main_config.bd_color, main_config.it_color,
-           main_config.ul_color, main_config.bl_color, main_config.co_color)) == NULL) {
+           main_config.cursor_bg_color, main_config.bd_color, main_config.ul_color,
+           main_config.bl_color, main_config.rv_color, main_config.it_color,
+           main_config.co_color)) == NULL) {
     goto error;
   }
 
@@ -746,12 +757,15 @@ static void open_pty(void *p, ui_screen_t *screen, char *dev) {
 #endif
     int ret;
 
+    encoding = main_config.encoding;
+    main_config.encoding = vt_term_get_encoding(screen->term);
+
     if ((new = create_term_intern()) == NULL) {
+      main_config.encoding = encoding;
+
       return;
     }
 
-    encoding = main_config.encoding;
-    main_config.encoding = vt_term_get_encoding(screen->term);
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
     if (!main_config.show_dialog) {
       default_server = main_config.default_server;
@@ -1066,9 +1080,6 @@ static int mlclient(void *self, ui_screen_t *screen, char *args,
     char *pty;
     int horizontal;
     char *sep;
-#if defined(USE_WIN32API) || defined(USE_LIBSSH2)
-    char **server_list;
-#endif
 
     if (argc >= 2 && *(argv[1]) != '-') {
       /*
@@ -1117,10 +1128,6 @@ static int mlclient(void *self, ui_screen_t *screen, char *args,
     orig_conf = main_config;
 
     ui_main_config_init(&main_config, conf, argc, argv);
-#if defined(USE_WIN32API) || defined(USE_LIBSSH2)
-    server_list = main_config.server_list;
-    main_config.server_list = orig_conf.server_list;
-#endif
 
     bl_conf_delete(conf);
 
@@ -1168,10 +1175,6 @@ static int mlclient(void *self, ui_screen_t *screen, char *args,
       }
     }
 
-#if defined(USE_WIN32API) || defined(USE_LIBSSH2)
-    orig_conf.server_list = main_config.server_list;
-    main_config.server_list = server_list;
-#endif
     ui_main_config_final(&main_config);
 
     main_config = orig_conf;
