@@ -13,6 +13,7 @@
 #include <pobl/bl_mem.h>
 #include <pobl/bl_str.h>  /* strdup */
 #include <pobl/bl_file.h> /* bl_file_set_cloexec */
+#include <pobl/bl_util.h> /* BL_SWAP */
 
 #include "../ui_window.h"
 #include "../ui_picture.h"
@@ -1115,6 +1116,11 @@ static int check_resize(u_int old_width, u_int old_height, int32_t *new_width, i
                         int check_inc) {
   u_int diff;
 
+  if (rotate_display) {
+    BL_SWAP(unsigned int, min_width, min_height);
+    BL_SWAP(unsigned int, width_inc, height_inc);
+  }
+
   if (old_width < *new_width) {
     diff = ((*new_width - old_width) / width_inc) * width_inc;
     if (check_inc || diff < width_inc) {
@@ -2059,7 +2065,15 @@ XID ui_display_get_group_leader(ui_display_t *disp) {
   }
 }
 
-void ui_display_rotate(int rotate) { rotate_display = rotate; }
+void ui_display_rotate(int rotate) {
+  if (num_displays > 0) {
+    bl_msg_printf("rotate_display option is not changeable.\n");
+
+    return;
+  }
+
+  rotate_display = rotate;
+}
 
 /* Don't call this internally from ui_display.c. Call resize_display(..., 0) instead. */
 int ui_display_resize(ui_display_t *disp, u_int width, u_int height) {
@@ -2290,14 +2304,14 @@ void ui_display_send_text_selection(ui_display_t *disp, XSelectionRequestEvent *
 }
 
 u_char ui_display_get_char(KeySym ksym) {
-    char buf[10];
-    int len;
+  char buf[10];
+  int len;
 
-    if ((len = xkb_keysym_to_utf8(ksym, buf, sizeof(buf))) > 0) {
-      return buf[0];
-    } else {
-      return 0;
-    }
+  if ((len = xkb_keysym_to_utf8(ksym, buf, sizeof(buf))) > 0) {
+    return buf[0];
+  } else {
+    return 0;
+  }
 }
 
 void ui_display_logical_to_physical_coordinates(ui_display_t *disp, int *x, int *y) {
@@ -2318,9 +2332,11 @@ void ui_display_logical_to_physical_coordinates(ui_display_t *disp, int *x, int
 #endif
 }
 
-void ui_display_set_maximized(ui_display_t *disp) {
+void ui_display_set_maximized(ui_display_t *disp, int flag) {
 #ifndef COMPAT_LIBVTE
-  wl_shell_surface_set_maximized(disp->display->shell_surface, disp->display->wlserv->output);
+  if (flag) {
+    wl_shell_surface_set_maximized(disp->display->shell_surface, disp->display->wlserv->output);
+  }
 #endif
 }
 
