@@ -5,7 +5,6 @@
 #include <stdio.h>
 #include <fcntl.h>    /* open */
 #include <unistd.h>   /* close */
-#include <sys/mman.h> /* mmap */
 #include <string.h>   /* memcmp */
 #include <sys/stat.h> /* fstat */
 #include <utime.h>    /* utime */
@@ -23,6 +22,13 @@
 #include <dirent.h>
 #endif
 
+#ifdef USE_WIN32API
+#define mmap(a, b, c, d, e, f) (NULL)
+#define munmap(a, b) (0)
+#else
+#include <sys/mman.h> /* mmap */
+#endif
+
 #ifdef USE_OT_LAYOUT
 #include <otl.h>
 #endif
@@ -64,7 +70,7 @@
 static XFontStruct **xfonts;
 static u_int num_xfonts;
 
-static XFontStruct *get_cached_xfont(const char *file, int32_t format);
+static XFontStruct *get_cached_xfont(const char *file, int32_t format, int is_aa);
 static int add_xfont_to_cache(XFontStruct *xfont);
 static void xfont_unref(XFontStruct *xfont);
 
@@ -520,7 +526,7 @@ static int load_pcf(XFontStruct *xfont, const char *file_path) {
       for (j = 0; j < xfont->height; j++) {
         u_char *line;
 
-        ui_get_bitmap_line(xfont, bitmap, j, line);
+        ui_get_bitmap_line(xfont, bitmap, j * xfont->glyph_width_bytes, line);
 
         for (i = 0; i < xfont->width; i++) {
           bl_msg_printf("%d", (line && ui_get_bitmap_cell(line, i)) ? 1 : 0);
@@ -575,9 +581,9 @@ static int is_pcf(const char *file_path) {
 /* 0 - 511 */
 #define SEG(idx) (((idx) >> 7) & 0x1ff)
 /* 0 - 127 */
-#define OFF(idx) ((idx)&0x7f)
+#define OFF(idx) ((idx) & 0x7f)
 /* +3 is for storing glyph position info. */
-#define IS_PROPORTIONAL(xfont) \
+#define HAS_POSITION_INFO_BY_GLYPH(xfont) \
   ((xfont)->glyph_size == (xfont)->glyph_width_bytes * (xfont)->height + 3)
 #define FONT_ROTATED (FONT_ITALIC << 1)
 
@@ -715,11 +721,11 @@ static int load_ft(XFontStruct *xfont, const char *file_path, int32_t format, in
   }
 
 face_found:
-  FT_Set_Pixel_Sizes(face, fontsize, fontsize);
+  if (face->units_per_EM == 0 /* units_per_EM can be 0 ("Noto Color Emoji") */) {
+    goto error;
+  }
 
-  xfont->format = format;
-  xfont->face = face;
-  xfont->is_aa = is_aa;
+  FT_Set_Pixel_Sizes(face, fontsize, fontsize);
 
   if (!load_glyph(face, format, get_glyph_index(face, 'M'), is_aa)) {
     bl_msg_printf("%s doesn't have outline glyphs.\n", file_path);
@@ -727,6 +733,9 @@ face_found:
     goto error;
   }
 
+  xfont->format = format;
+  xfont->face = face;
+  xfont->is_aa = is_aa;
   xfont->num_indeces = INITIAL_GLYPH_INDEX_TABLE_SIZE;
 
   if (!(xfont->file = strdup(file_path)) ||
@@ -832,6 +841,13 @@ static void clear_glyph_cache_ft(XFontStruct *xfont) {
   memset(xfont->glyph_indeces, 0, xfont->num_indeces * sizeof(u_int16_t));
 }
 
+static void enable_position_info_by_glyph(XFontStruct *xfont) {
+  /* Shrink glyph size (+1 is margin) */
+  xfont->glyph_width_bytes = (xfont->width_full / 2 + 1) * 3;
+  /* +3 is for storing glyph position info. */
+  xfont->glyph_size = xfont->glyph_width_bytes * xfont->height + 3;
+}
+
 static void unload_ft(XFontStruct *xfont) {
   FT_Face face;
   int count;
@@ -909,7 +925,7 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
     indeces = xfont->glyph_indeces;
   }
 
-  glyphs = xfont->glyphs;
+  glyphs = xfont->glyphs; /* Cast to u_char* to u_char** */
 
   if (!(idx = indeces[code])) {
     FT_Face face;
@@ -962,6 +978,7 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
       }
     }
 
+    /* idx can be changed if (pitch + left_pitch - 1) / xfont->glyph_width_bytes > 0. */
     idx = ++xfont->num_glyphs;
 
 #if 0
@@ -1017,7 +1034,33 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
     if (xfont->is_aa) {
       left_pitch *= 3;
 
-      if (face->glyph->bitmap.pitch < xfont->glyph_width_bytes) {
+      if (HAS_POSITION_INFO_BY_GLYPH(xfont)) {
+        int additional_glyphs;
+        int count;
+
+        pitch = face->glyph->bitmap.pitch;
+
+        additional_glyphs = (pitch + left_pitch - 1) / xfont->glyph_width_bytes;
+#if 0
+        bl_debug_printf("Use %d(=(%d+%d-1)/%d) glyph slots for one glyph.\n",
+                        1 + additional_glyphs, pitch, left_pitch, xfont->glyph_width_bytes);
+#endif
+        for (count = 0; count < additional_glyphs; count++) {
+          if (OFF(xfont->num_glyphs + count) == 0) {
+            if (1 + additional_glyphs > GLYPH_TABLE_SIZE ||
+                !(glyphs[SEG(xfont->num_glyphs + count)] =
+                    calloc(GLYPH_TABLE_SIZE, xfont->glyph_size))) {
+              xfont->num_glyphs--;
+              indeces[code] = 0;
+
+              return NULL;
+            }
+            xfont->num_glyphs += (1 + count);
+            indeces[code] = idx = xfont->num_glyphs;
+          }
+        }
+        xfont->num_glyphs += additional_glyphs;
+      } else if (face->glyph->bitmap.pitch < xfont->glyph_width_bytes) {
         pitch = face->glyph->bitmap.pitch;
 
         if (pitch + left_pitch > xfont->glyph_width_bytes) {
@@ -1046,39 +1089,42 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
     dst = glyph + (xfont->glyph_width_bytes * y);
 
     if (xfont->is_aa) {
-      for (y = 0; y < rows; y++) {
-        memcpy(dst + left_pitch, src, pitch);
-
-        src += face->glyph->bitmap.pitch;
-        dst += xfont->glyph_width_bytes;
-      }
-
-      if (IS_PROPORTIONAL(xfont)) {
+      if (HAS_POSITION_INFO_BY_GLYPH(xfont)) {
         /* Storing glyph position info. (ISCII or ISO10646_UCS4_1_V) */
 
-        dst = glyph + xfont->glyph_size - 3;
-
-        dst[0] = (face->glyph->advance.x >> 6);                /* advance */
-        dst[2] = (face->glyph->bitmap.width + left_pitch) / 3; /* width */
-        if (dst[2] > xfont->width_full) {
-          dst[2] = xfont->width_full; /* == glyph_width_bytes / 3 */
-        }
+        glyph[0] = (face->glyph->advance.x >> 6);                /* advance */
+        glyph[2] = (face->glyph->bitmap.width + left_pitch) / 3; /* width */
 
         if (face->glyph->bitmap_left < 0) {
-          dst[1] = -face->glyph->bitmap_left; /* retreat */
+          glyph[1] = -face->glyph->bitmap_left; /* retreat */
         } else {
-          dst[1] = 0;
+          glyph[1] = 0;
         }
 
-        if (dst[0] == 0 && dst[2] > dst[0] + dst[1]) {
-          dst[1] = dst[2] - dst[0]; /* retreat */
+        if (glyph[0] == 0 && glyph[2] > glyph[0] + glyph[1]) {
+          glyph[1] = glyph[2] - glyph[0]; /* retreat */
         }
 
+        dst = glyph + glyph[2] * 3 * y + 3;
+
 #if 0
         bl_debug_printf("%x %c A %d R %d W %d-> A %d R %d W %d\n", code, code,
                         face->glyph->advance.x >> 6, face->glyph->bitmap_left,
-                        face->glyph->bitmap.width, dst[0], dst[1], dst[2]);
+                        face->glyph->bitmap.width, glyph[0], glyph[1], glyph[2]);
 #endif
+        for (y = 0; y < rows; y++) {
+          memcpy(dst + left_pitch, src, pitch);
+
+          src += face->glyph->bitmap.pitch;
+          dst += (glyph[2] * 3);
+        }
+      } else {
+        for (y = 0; y < rows; y++) {
+          memcpy(dst + left_pitch, src, pitch);
+
+          src += face->glyph->bitmap.pitch;
+          dst += xfont->glyph_width_bytes;
+        }
       }
     } else {
       int shift;
@@ -1117,9 +1163,9 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
 }
 
 static int load_xfont(XFontStruct *xfont, const char *file_path, int32_t format,
-                      u_int bytes_per_pixel, ef_charset_t cs, int noaa) {
+                      ef_charset_t cs, int is_aa) {
   if (!is_pcf(file_path)) {
-    return load_ft(xfont, file_path, format, (bytes_per_pixel > 1) && !noaa, 0);
+    return load_ft(xfont, file_path, format, is_aa, 0);
   } else {
     return load_pcf(xfont, file_path);
   }
@@ -1404,15 +1450,43 @@ static FcPattern *parse_font_name(const char *fontname, int *is_bold, int *is_it
   if ((pattern = fc_pattern_create(family))) {
     match = FcFontMatch(NULL, pattern, &result);
 
-    if (IS_ISCII(cs) && !check_iscii_font(match)) {
+    if (IS_ISCII(cs) && match && !check_iscii_font(match)) {
       FcPatternDestroy(match);
       FcPatternDestroy(pattern);
 
       return NULL;
     }
 
-    if (compl_pattern == NULL) {
+    if (compl_pattern == NULL && match) {
       num_fc_files = strip_pattern((compl_pattern = pattern), match);
+
+      /*
+       * XXX
+       * FcFontList() is not used.
+       * If you want to match complementary fonts, add <match target="pattern">...</match>
+       * to ~/.fonts.conf
+       */
+#if 0
+      {
+        FcPattern *pat = FcPatternCreate();
+        FcObjectSet *objset = FcObjectSetBuild(FC_FAMILY, NULL);
+        FcFontSet *fontset = FcFontList(NULL, pat, objset);
+        FcValue val;
+        int count;
+
+        for (count = 0; count < fontset->nfont; count++) {
+          FcPatternGet(fontset->fonts[count], FC_FAMILY, 0, &val);
+
+          if (!is_same_family(compl_pattern, val.u.s)) {
+            FcPatternAdd(compl_pattern, FC_FAMILY, val, FcTrue /* append */);
+            num_fc_files++;
+#if 0
+            bl_debug_printf("Add Font List %s [%]\n", val.u.s);
+#endif
+          }
+        }
+      }
+#endif
     } else {
       FcPatternDestroy(pattern);
     }
@@ -1423,6 +1497,27 @@ static FcPattern *parse_font_name(const char *fontname, int *is_bold, int *is_it
   return match;
 }
 
+#ifdef USE_WIN32API
+static void init_fontconfig(void) {
+  if (!getenv("FONTCONFIG_PATH") && !getenv("FONTCONFIG_FILE")) {
+    /*
+     * See fontconfig-x.x.x/src/fccfg.c
+     * (DllMain(), FcConfigFileExists(), FcConfigGetPath() and FcConfigFilename())
+     *
+     * [commant in DllMain]
+     * If the fontconfig DLL is in a "bin" or "lib" subfolder, assume it's a Unix-style
+     * installation tree, and use "etc/fonts" in there as FONTCONFIG_PATH.
+     * Otherwise use the folder where the DLL is as FONTCONFIG_PATH.
+     *
+     * [comment in FcConfigFileExists]
+     * make sure there's a single separator
+     * (=> If FONTCONFIG_PATH="", FONTCONFIG_FILE="/...")
+     */
+    putenv("FONTCONFIG_PATH=.");
+  }
+}
+#endif
+
 #endif /* USE_FONTCONFIG */
 
 static u_char *get_ft_bitmap(XFontStruct *xfont, u_int32_t ch, int use_ot_layout,
@@ -1440,9 +1535,7 @@ static u_char *get_ft_bitmap(XFontStruct *xfont, u_int32_t ch, int use_ot_layout
     }
 
     if ((code = get_glyph_index(xfont->face, ch)) == 0) {
-      if (!IS_PROPORTIONAL(xfont)) {
-        goto compl_font;
-      }
+      goto compl_font;
     }
   } else {
     code = ch;
@@ -1509,7 +1602,7 @@ static u_char *get_ft_bitmap(XFontStruct *xfont, u_int32_t ch, int use_ot_layout
       XFontStruct *compl;
 
       if (!xfont->compl_xfonts[count]) {
-        if (!(compl = get_cached_xfont(fc_files[count], xfont->format))) {
+        if (!(compl = get_cached_xfont(fc_files[count], xfont->format, xfont->is_aa))) {
           if (!(compl = calloc(1, sizeof(XFontStruct)))) {
             continue;
           }
@@ -1531,12 +1624,19 @@ static u_char *get_ft_bitmap(XFontStruct *xfont, u_int32_t ch, int use_ot_layout
         }
 
         xfont->compl_xfonts[count] = compl;
+      } else {
+        compl = xfont->compl_xfonts[count];
       }
 
-      if ((code = get_glyph_index(xfont->compl_xfonts[count]->face, ch)) > 0) {
-        if ((bitmap = get_ft_bitmap_intern(xfont->compl_xfonts[count], code, ch))) {
+      if (HAS_POSITION_INFO_BY_GLYPH(xfont) && !HAS_POSITION_INFO_BY_GLYPH(compl)) {
+        clear_glyph_cache_ft(compl);
+        enable_position_info_by_glyph(compl);
+      }
+
+      if ((code = get_glyph_index(compl->face, ch)) > 0) {
+        if ((bitmap = get_ft_bitmap_intern(compl, code, ch))) {
           if (compl_xfont) {
-            *compl_xfont = xfont->compl_xfonts[count];
+            *compl_xfont = compl;
           }
 #ifdef __DEBUG
           bl_debug_printf("Use %s font to show U+%x\n", (*compl_xfont)->file, ch);
@@ -1555,18 +1655,19 @@ static u_char *get_ft_bitmap(XFontStruct *xfont, u_int32_t ch, int use_ot_layout
 
 #else /* USE_FREETYPE */
 
-#define load_xfont(xfont, file_path, format, bytes_per_pixel, cs, noaa) load_pcf(xfont, file_path)
+#define load_xfont(xfont, file_path, format, cs, is_aa) load_pcf(xfont, file_path)
 #define unload_xfont(xfont) unload_pcf(xfont)
 
 #endif /* USE_FREETYPE */
 
-static XFontStruct *get_cached_xfont(const char *file, int32_t format) {
+static XFontStruct *get_cached_xfont(const char *file, int32_t format, int is_aa) {
   u_int count;
 
   for (count = 0; count < num_xfonts; count++) {
     if (strcmp(xfonts[count]->file, file) == 0
 #ifdef USE_FREETYPE
         && (xfonts[count]->face == NULL || xfonts[count]->format == format)
+        && xfonts[count]->is_aa == is_aa
 #endif
         ) {
 #ifdef DEBUG
@@ -1661,6 +1762,7 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
   u_int format;
 #endif
   u_int cols;
+  int is_aa;
 
   cs = FONT_CS(id);
 
@@ -1671,6 +1773,10 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
     int is_bold;
     int is_italic;
 
+#ifdef USE_WIN32API
+    init_fontconfig();
+#endif
+
     if (!(pattern = parse_font_name(fontname, &is_bold, &is_italic, &percent, cs))) {
       return NULL;
     }
@@ -1797,7 +1903,17 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
       return NULL;
     }
 
-    font_file = bl_str_sep(&percent_str, ":");
+#ifdef USE_WIN32API
+    if (percent_str[0] != '\0' && percent_str[1] == ':') {
+      /* c:/Users/... */
+      font_file = percent_str;
+      percent_str += 2;
+      bl_str_sep(&percent_str, ":");
+    } else
+#endif
+    {
+      font_file = bl_str_sep(&percent_str, ":");
+    }
 
     if (!percent_str || !bl_str_to_uint(&percent, percent_str)) {
       percent = 0;
@@ -1837,13 +1953,19 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
     sprintf(decsp_id, "decsp-%dx%d", col_width, fontsize);
   }
 
+  if (!(font_present & FONT_NOAA) && display->bytes_per_pixel > 1) {
+    is_aa = 1;
+  } else {
+    is_aa = 0;
+  }
+
   if ((font->xfont = get_cached_xfont(decsp_id ? decsp_id : font_file,
 #ifdef USE_FREETYPE
-                                      format
+                                      format,
 #else
-                                      0
+                                      0,
 #endif
-       ))) {
+                                      is_aa))) {
     goto xfont_loaded;
   }
 
@@ -1871,8 +1993,7 @@ ui_font_t *ui_font_new(Display *display, vt_font_t id, int size_attr, ui_type_en
                          font_file, fontsize, col_width,
                          use_medium_for_bold, letter_space);
     }
-  } else if (!load_xfont(font->xfont, font_file, format, display->bytes_per_pixel, cs,
-                         font_present & FONT_NOAA)) {
+  } else if (!load_xfont(font->xfont, font_file, format, cs, is_aa)) {
     bl_msg_printf("Failed to load %s.\n", font_file);
 
     free(font->xfont);
@@ -1909,24 +2030,23 @@ xfont_loaded:
     cols = 1;
   }
 
-/*
- * font->is_var_col_width == false and font->is_proportional == true
- * is impossible on framebuffer.
- */
 #ifdef USE_FREETYPE
+  if (HAS_POSITION_INFO_BY_GLYPH(font->xfont)) {
+    font->is_proportional = 1;
+  }
+
   if (IS_ISCII(cs) && font->xfont->is_aa &&
-      (font->xfont->ref_count == 1 || IS_PROPORTIONAL(font->xfont))) {
+      (font->xfont->ref_count == 1 || HAS_POSITION_INFO_BY_GLYPH(font->xfont))) {
     /* Proportional glyph is available on ISCII alone for now. */
     font->is_var_col_width = font->is_proportional = 1;
 
     if (font->xfont->ref_count == 1) {
       init_iscii_ft(font->xfont->face);
-      /* +3 is for storing glyph position info. */
-      font->xfont->glyph_size += 3;
+      enable_position_info_by_glyph(font->xfont);
     }
   } else
 #endif
-      if (font_present & FONT_VAR_WIDTH) {
+  if (font_present & FONT_VAR_WIDTH) {
     /*
      * If you use fixed-width fonts whose width is differnet from
      * each other.
@@ -1934,18 +2054,13 @@ xfont_loaded:
     font->is_var_col_width = 1;
 
 #ifdef USE_FREETYPE
-    if (cs == ISO10646_UCS4_1_V) {
-      font->is_proportional = 1;
+    font->is_proportional = 1;
 
-      if (font->xfont->ref_count == 1) {
-        /* +3 is for storing glyph position info. */
-        font->xfont->glyph_size += 3;
-      } else if (!IS_PROPORTIONAL(font->xfont)) {
-        clear_glyph_cache_ft(font->xfont);
-
-        /* +3 is for storing glyph position info. */
-        font->xfont->glyph_size += 3;
-      }
+    if (font->xfont->ref_count == 1) {
+      enable_position_info_by_glyph(font->xfont);
+    } else if (!HAS_POSITION_INFO_BY_GLYPH(font->xfont)) {
+      clear_glyph_cache_ft(font->xfont);
+      enable_position_info_by_glyph(font->xfont);
     }
 #endif
   }
@@ -2133,11 +2248,21 @@ int ui_font_has_ot_layout_table(ui_font_t *font) {
 #ifdef USE_FREETYPE
   if (font->xfont->face) {
     if (!font->ot_font) {
-      if (font->ot_font_not_found || !(font->ot_font = otl_open(font->xfont->face, 0))) {
+      if (font->ot_font_not_found) {
+        return 0;
+      }
+
+      if (!(font->ot_font = otl_open(font->xfont->face, 0))) {
         font->ot_font_not_found = 1;
 
         return 0;
       }
+
+      if (!HAS_POSITION_INFO_BY_GLYPH(font->xfont)) {
+        clear_glyph_cache_ft(font->xfont);
+        enable_position_info_by_glyph(font->xfont);
+      }
+      font->is_proportional = 1; /* font->is_var_col_width can be 0 or 1. */
     }
 
     return 1;
@@ -2162,7 +2287,7 @@ u_int ui_calculate_char_width(ui_font_t *font, u_int32_t ch, ef_charset_t cs, in
   }
 
 #if defined(USE_FREETYPE)
-  if (font->xfont->is_aa && font->is_proportional) {
+  if (font->xfont->is_aa && font->is_proportional && font->is_var_col_width) {
     u_char *glyph;
 
     if ((glyph = get_ft_bitmap(font->xfont, ch,
@@ -2172,7 +2297,7 @@ u_int ui_calculate_char_width(ui_font_t *font, u_int32_t ch, ef_charset_t cs, in
                                0
 #endif
                                , NULL))) {
-      return glyph[font->xfont->glyph_size - 3];
+      return glyph[0];
     }
   }
 #endif
