@@ -29,9 +29,18 @@
 
 static ui_display_t _disp; /* Singleton */
 static Display _display;
+#ifdef USE_WIN32API
+static DWORD main_tid; /* XXX set in main(). */
+#endif
 
 /* --- static functions --- */
 
+#ifdef USE_WIN32API
+static VOID CALLBACK timer_proc(HWND hwnd, UINT msg, UINT timerid, DWORD time) {
+  ui_display_idling(&_disp);
+}
+#endif
+
 static LRESULT CALLBACK window_proc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam) {
   XEvent event;
   int count;
@@ -88,6 +97,15 @@ ui_display_t *ui_display_open(char *disp_name, /* Ignored */
     return &_disp;
   }
 
+#ifdef USE_WIN32API
+  /*
+   * XXX
+   * vt_pty_ssh_new() isn't called from the main thread, so main_tid
+   * must be set here, not in vt_pty_ssh_new().
+   */
+  main_tid = GetCurrentThreadId();
+#endif
+
   /* Callback should be set before bl_dialog() is called. */
   bl_dialog_set_callback(dialog_cb);
 
@@ -139,6 +157,11 @@ ui_display_t *ui_display_open(char *disp_name, /* Ignored */
   _display.fd = fd;
   _disp.display = &_display;
 
+#ifdef USE_WIN32API
+  /* ui_window_manager_idling() called in 0.1sec. */
+  SetTimer(NULL, 0, 100, timer_proc);
+#endif
+
   return &_disp;
 }
 
@@ -353,3 +376,10 @@ Cursor ui_display_get_cursor(ui_display_t *disp, u_int shape) {
 }
 
 XID ui_display_get_group_leader(ui_display_t *disp) { return None; }
+
+#ifdef USE_WIN32API
+void ui_display_trigger_pty_read(void) {
+  /* Exit GetMessage() in x_display_receive_next_event(). */
+  PostThreadMessage(main_tid, WM_APP, 0, 0);
+}
+#endif
