@@ -6,7 +6,7 @@
 #include <pobl/bl_str.h>    /* bl_snprintf */
 #include <pobl/bl_mem.h>    /* alloca */
 #include <pobl/bl_str.h>    /* bl_str_sep/bl_str_to_int/memset/strncasecmp */
-#include <pobl/bl_util.h>   /* DIGIT_STR_LEN/K_MIN */
+#include <pobl/bl_util.h>   /* DIGIT_STR_LEN */
 #include <pobl/bl_locale.h> /* bl_get_lang() */
 #include <mef/ef_ucs4_map.h>
 #include <mef/ef_ucs_property.h>
@@ -594,6 +594,20 @@ static int parse_xfont_name(char **font_xlfd, char **percent, /* NULL can be ret
   return 1;
 }
 
+static XFontStruct *load_xfont_intern(Display *display, char *fontname, size_t max_len,
+                                      const char *family, const char *weight,
+                                      const char *slant, const char *width, const char *additional,
+                                      u_int fontsize, const char *spacing, const char *encoding) {
+  bl_snprintf(fontname, max_len, "-*-%s-%s-%s-%s-%s-%d-*-*-*-%s-*-%s", family, weight, slant,
+              width, additional, fontsize, spacing, encoding);
+
+#ifdef DEBUG
+  bl_debug_printf(BL_DEBUG_TAG " loading %s.\n", fontname);
+#endif
+
+  return XLoadQueryFont(display, fontname);
+}
+
 static XFontStruct *load_xfont(Display *display, const char *family, const char *weight,
                                const char *slant, const char *width, u_int fontsize,
                                const char *spacing, const char *encoding) {
@@ -601,66 +615,41 @@ static XFontStruct *load_xfont(Display *display, const char *family, const char
   char *fontname;
   size_t max_len;
 
-  /* "+ 20" means the num of '-' , '*'(19byte) and null chars. */
-  max_len = 3 /* gnu */ + strlen(family) + 7 /* unifont */ + strlen(weight) + strlen(slant) +
+  /* "+ 19" means the num of '-' , '*'(18byte) and null chars. */
+  max_len = strlen(family) + 7 /* unifont */ + strlen(weight) + strlen(slant) +
             strlen(width) + 2 /* lang */ + DIGIT_STR_LEN(fontsize) + strlen(spacing) +
-            strlen(encoding) + 20;
+            strlen(encoding) + 19;
 
   if ((fontname = alloca(max_len)) == NULL) {
     return NULL;
   }
 
-  bl_snprintf(fontname, max_len, "-*-%s-%s-%s-%s--%d-*-*-*-%s-*-%s", family, weight, slant, width,
-              fontsize, spacing, encoding);
-
-#ifdef DEBUG
-  bl_debug_printf(BL_DEBUG_TAG " loading %s.\n", fontname);
-#endif
-
-  if ((xfont = XLoadQueryFont(display, fontname))) {
+  if ((xfont = load_xfont_intern(display, fontname, max_len, family, weight, slant, width, "*",
+                                 fontsize, spacing, encoding))) {
     return xfont;
   }
 
   if (strcmp(encoding, "iso10646-1") == 0 && strcmp(family, "biwidth") == 0) {
     /* XFree86 Unicode font */
 
-    bl_snprintf(fontname, max_len, "-*-*-%s-%s-%s-%s-%d-*-*-*-%s-*-%s", weight, slant, width,
-                bl_get_lang(), fontsize, spacing, encoding);
-
-#ifdef DEBUG
-    bl_debug_printf(BL_DEBUG_TAG " loading %s.\n", fontname);
-#endif
-
-    if ((xfont = XLoadQueryFont(display, fontname))) {
+    if ((xfont = load_xfont_intern(display, fontname, max_len, "*", weight, slant, width,
+                                   bl_get_lang(), fontsize, spacing, encoding))) {
       return xfont;
     }
 
-    if (strcmp(bl_get_lang(), "ja") != 0) {
-      bl_snprintf(fontname, max_len, "-*-*-%s-%s-%s-ja-%d-*-*-*-%s-*-%s", weight, slant, width,
-                  fontsize, spacing, encoding);
-
-#ifdef DEBUG
-      bl_debug_printf(BL_DEBUG_TAG " loading %s.\n", fontname);
-#endif
-
-      if ((xfont = XLoadQueryFont(display, fontname))) {
-        return xfont;
-      }
+    if (strcmp(bl_get_lang(), "ja") != 0 &&
+        (xfont = load_xfont_intern(display, fontname, max_len, "*", weight, slant, width,
+                                   "ja", fontsize, spacing, encoding))) {
+      return xfont;
     }
 
-    /* GNU Unifont */
+    /* GNU Unifont (-gnu-unifont) */
 
-    bl_snprintf(fontname, max_len, "-gnu-unifont-%s-%s-%s--%d-*-*-*-%s-*-%s", weight, slant, width,
-                fontsize, spacing, encoding);
-
-#ifdef DEBUG
-    bl_debug_printf(BL_DEBUG_TAG " loading %s.\n", fontname);
-#endif
-
-    return XLoadQueryFont(display, fontname);
-  } else {
-    return NULL;
+    return load_xfont_intern(display, fontname, max_len, "unifont", weight, slant,
+                             width, "*", fontsize, spacing, encoding);
   }
+
+  return NULL;
 }
 
 static int xcore_set_font(ui_font_t *font, const char *fontname, u_int fontsize,
@@ -769,7 +758,7 @@ static int xcore_set_font(ui_font_t *font, const char *fontname, u_int fontsize,
     num_spacings = 1;
   }
 
-  for (count = 0;; count++) {
+  for (count = 0; ; count++) {
     FOREACH_FONT_ENCODINGS(csinfo, font_encoding_p) {
       int idx;
 
@@ -1024,15 +1013,15 @@ static int xft_set_font(ui_font_t *font, const char *fontname, u_int fontsize,
                         u_int col_width, /* if usascii font wants to be set , 0 will be set. */
                         u_int letter_space,
                         int aa_opt, /* 0 = default , 1 = enable , -1 = disable */
-                        int use_point_size_for_fc, double dpi_for_fc) {
+                        int use_point_size, double dpi) {
   int (*func)(ui_font_t *, const char *, u_int, u_int, u_int, int, int, double);
 
   if (!(func = ui_load_type_xft_func(UI_SET_FONT))) {
     return 0;
   }
 
-  return (*func)(font, fontname, fontsize, col_width, letter_space, aa_opt, use_point_size_for_fc,
-                 dpi_for_fc);
+  return (*func)(font, fontname, fontsize, col_width, letter_space, aa_opt, use_point_size,
+                 dpi);
 }
 
 static int xft_set_ot_font(ui_font_t *font) {
@@ -1063,7 +1052,7 @@ static u_int xft_convert_text_to_glyphs(ui_font_t *font, u_int32_t *shaped, u_in
 #elif defined(USE_TYPE_XFT)
 u_int xft_calculate_char_width(ui_font_t *font, u_int32_t ch);
 int xft_set_font(ui_font_t *font, const char *fontname, u_int fontsize, u_int col_width,
-                 u_int letter_space, int aa_opt, int use_point_size_for_fc, double dpi_for_fc);
+                 u_int letter_space, int aa_opt, int use_point_size, double dpi);
 int xft_set_ot_font(ui_font_t *font);
 #define xft_convert_text_to_glyphs(font, shaped, shaped_len, offsets, widths, cmapped, src,   \
                                    src_len, script, features)                                 \
@@ -1091,15 +1080,14 @@ static int cairo_set_font(ui_font_t *font, const char *fontname, u_int fontsize,
                           u_int col_width, /* if usascii font wants to be set , 0 will be set. */
                           u_int letter_space,
                           int aa_opt, /* 0 = default , 1 = enable , -1 = disable */
-                          int use_point_size_for_fc, double dpi_for_fc) {
+                          int use_point_size, double dpi) {
   int (*func)(ui_font_t *, const char *, u_int, u_int, u_int, int, int, double);
 
   if (!(func = ui_load_type_cairo_func(UI_SET_FONT))) {
     return 0;
   }
 
-  return (*func)(font, fontname, fontsize, col_width, letter_space, aa_opt, use_point_size_for_fc,
-                 dpi_for_fc);
+  return (*func)(font, fontname, fontsize, col_width, letter_space, aa_opt, use_point_size, dpi);
 }
 
 static int cairo_set_ot_font(ui_font_t *font) {
@@ -1130,7 +1118,7 @@ static u_int cairo_convert_text_to_glyphs(ui_font_t *font, u_int32_t *shaped, u_
 #elif defined(USE_TYPE_CAIRO)
 u_int cairo_calculate_char_width(ui_font_t *font, u_int32_t ch);
 int cairo_set_font(ui_font_t *font, const char *fontname, u_int fontsize, u_int col_width,
-                   u_int letter_space, int aa_opt, int use_point_size_for_fc, double dpi_for_fc);
+                   u_int letter_space, int aa_opt, int use_point_size, double dpi);
 int cairo_set_ot_font(ui_font_t *font);
 #define cairo_convert_text_to_glyphs(font, shaped, shaped_len, offsets, widths, cmapped, src, \
                                      src_len, script, features)                               \
@@ -1398,7 +1386,13 @@ int ui_font_has_ot_layout_table(ui_font_t *font) {
   if (font->cairo_font) {
 #ifdef USE_OT_LAYOUT
     if (!font->ot_font) {
-      if (font->ot_font_not_found || !cairo_set_ot_font(font)) {
+      if (font->ot_font_not_found) {
+        return 0;
+      }
+
+      if (!cairo_set_ot_font(font)) {
+        font->ot_font_not_found = 1;
+
         return 0;
       }
     }
@@ -1411,7 +1405,13 @@ int ui_font_has_ot_layout_table(ui_font_t *font) {
   if (font->xft_font) {
 #ifdef USE_OT_LAYOUT
     if (!font->ot_font) {
-      if (font->ot_font_not_found || !xft_set_ot_font(font)) {
+      if (font->ot_font_not_found) {
+        return 0;
+      }
+
+      if (!xft_set_ot_font(font)) {
+        font->ot_font_not_found = 1;
+
         return 0;
       }
     }
