@@ -245,17 +245,16 @@ static void value_table_refresh(u_char *value_table, /* 256 bytes */
 }
 
 static int modify_pixmap(ui_display_t *disp, Pixmap src_pixmap,
-                         Pixmap dst_pixmap,             /* Can be same as src_pixmap */
+                         Pixmap dst_pixmap /* Can be same as src_pixmap */,
+                         u_int width, u_int height,
                          ui_picture_modifier_t *pic_mod /* Mustn't be normal */
                          ) {
   u_char value_table[256];
   u_int x, y;
-  u_int width, height;
   XImage *image;
   u_char r, g, b;
   u_long pixel;
 
-  get_drawable_size(disp->display, src_pixmap, &width, &height);
   if ((image = XGetImage(disp->display, src_pixmap, 0, 0, width, height, AllPlanes, ZPixmap)) ==
       NULL) {
     return 0;
@@ -455,6 +454,7 @@ static int load_sixel(ui_display_t *disp, char *path, Pixmap *pixmap,
 static GdkPixbuf *load_file(char *path,   /* If NULL is specified, cache is cleared. */
                             u_int width,  /* 0 == image width */
                             u_int height, /* 0 == image height */
+                            int keep_aspect,
                             GdkInterpType scale_type) {
   static char *name = NULL;
   static GdkPixbuf *orig_cache = NULL;
@@ -520,6 +520,20 @@ static GdkPixbuf *load_file(char *path,   /* If NULL is specified, cache is clea
     height = gdk_pixbuf_get_height(pixbuf);
   }
 
+  if (keep_aspect) {
+    u_int w = height * gdk_pixbuf_get_width(pixbuf) / gdk_pixbuf_get_height(pixbuf);
+
+    if (w < width) {
+      width = w;
+    } else {
+      u_int h = width * gdk_pixbuf_get_height(pixbuf) / gdk_pixbuf_get_width(pixbuf);
+
+      if (h < height) {
+        height = h;
+      }
+    }
+  }
+
   /* It is necessary to scale orig_cache if width/height don't correspond. */
   if ((width != gdk_pixbuf_get_width(pixbuf)) || (height != gdk_pixbuf_get_height(pixbuf))) {
     if (pixbuf != orig_cache) {
@@ -1081,13 +1095,13 @@ static int modify_image(GdkPixbuf *pixbuf, ui_picture_modifier_t *pic_mod /* Mus
 
 #ifdef NO_TOOLS
 
-#define load_file(disp, width, height, path, pic_mod, pixmap, mask) (0)
+#define load_file(disp, path, width, height, keep_aspect, pic_mod, pixmap, mask) (0)
 #define create_cardinals_from_file(path, width, height) (NULL)
 
 #else /* NO_TOOLS */
 
-static pid_t exec_mlimgloader(int *read_fd, int *write_fd, Window window, u_int width, u_int height,
-                              char *path, char *cardinal_opt) {
+static pid_t exec_mlimgloader(int *read_fd, int *write_fd, Window window,
+                              u_int width, u_int height, char *path, char *dst, char *opt) {
   int fds1[2];
   int fds2[2];
   pid_t pid;
@@ -1107,7 +1121,7 @@ static pid_t exec_mlimgloader(int *read_fd, int *write_fd, Window window, u_int
   if (pid == 0) {
     /* child process */
 
-    char *args[7];
+    char *args[8];
     char win_str[DIGIT_STR_LEN(Window) + 1];
     char width_str[DIGIT_STR_LEN(u_int) + 1];
     char height_str[DIGIT_STR_LEN(u_int) + 1];
@@ -1120,8 +1134,9 @@ static pid_t exec_mlimgloader(int *read_fd, int *write_fd, Window window, u_int
     sprintf(height_str, "%u", height);
     args[3] = height_str;
     args[4] = path;
-    args[5] = cardinal_opt;
-    args[6] = NULL;
+    args[5] = dst;
+    args[6] = opt;
+    args[7] = NULL;
 
     close(fds1[1]);
     close(fds2[0]);
@@ -1153,7 +1168,7 @@ error1:
   return -1;
 }
 
-static int load_file(ui_display_t *disp, u_int width, u_int height, char *path,
+static int load_file(ui_display_t *disp, char *path, u_int *width, u_int *height, int keep_aspect,
                      ui_picture_modifier_t *pic_mod, Pixmap *pixmap, Pixmap *mask /* Can be NULL */
                      ) {
   int read_fd;
@@ -1163,8 +1178,8 @@ static int load_file(ui_display_t *disp, u_int width, u_int height, char *path,
   Pixmap mask_tmp;
   ssize_t size;
 
-  if (exec_mlimgloader(&read_fd, &write_fd, ui_display_get_group_leader(disp), width, height, path,
-                       NULL) == -1) {
+  if (exec_mlimgloader(&read_fd, &write_fd, ui_display_get_group_leader(disp), *width, *height,
+                       path, "pixmap", keep_aspect ? "-a" : NULL) == -1) {
     return 0;
   }
 
@@ -1182,17 +1197,17 @@ static int load_file(ui_display_t *disp, u_int width, u_int height, char *path,
   bl_debug_printf(BL_DEBUG_TAG " Receiving pixmap %lu %lu\n", pixmap_tmp, mask_tmp);
 #endif
 
-  if (width == 0 || height == 0) {
-    get_drawable_size(disp->display, pixmap_tmp, &width, &height);
+  if (*width == 0 || *height == 0 || keep_aspect) {
+    get_drawable_size(disp->display, pixmap_tmp, width, height);
   }
 
   *pixmap =
-      XCreatePixmap(disp->display, ui_display_get_group_leader(disp), width, height, disp->depth);
+      XCreatePixmap(disp->display, ui_display_get_group_leader(disp), *width, *height, disp->depth);
 
   if (!ui_picture_modifier_is_normal(pic_mod)) {
-    modify_pixmap(disp, pixmap_tmp, *pixmap, pic_mod);
+    modify_pixmap(disp, pixmap_tmp, *pixmap, *width, *height, pic_mod);
   } else {
-    XCopyArea(disp->display, pixmap_tmp, *pixmap, disp->gc->gc, 0, 0, width, height, 0, 0);
+    XCopyArea(disp->display, pixmap_tmp, *pixmap, disp->gc->gc, 0, 0, *width, *height, 0, 0);
   }
 
   if (mask) {
@@ -1200,9 +1215,9 @@ static int load_file(ui_display_t *disp, u_int width, u_int height, char *path,
       GC mask_gc;
       XGCValues gcv;
 
-      *mask = XCreatePixmap(disp->display, ui_display_get_group_leader(disp), width, height, 1);
+      *mask = XCreatePixmap(disp->display, ui_display_get_group_leader(disp), *width, *height, 1);
       mask_gc = XCreateGC(disp->display, *mask, 0, &gcv);
-      XCopyArea(disp->display, mask_tmp, *mask, mask_gc, 0, 0, width, height, 0, 0);
+      XCopyArea(disp->display, mask_tmp, *mask, mask_gc, 0, 0, *width, *height, 0, 0);
 
       XFreeGC(disp->display, mask_gc);
     } else {
@@ -1231,7 +1246,7 @@ static u_int32_t *create_cardinals_from_file(char *path, u_int32_t width, u_int3
   u_int32_t *cardinal;
   ssize_t size;
 
-  if (exec_mlimgloader(&read_fd, &write_fd, None, width, height, path, "-c") == -1) {
+  if (exec_mlimgloader(&read_fd, &write_fd, None, width, height, path, "stdout", NULL) == -1) {
     return 0;
   }
 
@@ -1293,7 +1308,7 @@ void ui_imagelib_display_closed(Display *display) {
   if (display_count == 0) {
 #ifdef BUILTIN_IMAGELIB
     /* drop pixbuf cache */
-    load_file(NULL, 0, 0, 0);
+    load_file(NULL, 0, 0, 0, 0);
 #endif
   }
 }
@@ -1309,6 +1324,9 @@ Pixmap ui_imagelib_load_file_for_background(ui_window_t *win, char *path,
                                             ui_picture_modifier_t *pic_mod) {
 #ifdef BUILTIN_IMAGELIB
   GdkPixbuf *pixbuf;
+#else
+  u_int width;
+  u_int height;
 #endif
   Pixmap pixmap;
 
@@ -1326,7 +1344,7 @@ Pixmap ui_imagelib_load_file_for_background(ui_window_t *win, char *path,
 
 #ifdef BUILTIN_IMAGELIB
 
-  if (!(pixbuf = load_file(path, ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win), GDK_INTERP_BILINEAR))) {
+  if (!(pixbuf = load_file(path, ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win), 0, GDK_INTERP_BILINEAR))) {
     return None;
   }
 
@@ -1374,7 +1392,9 @@ error:
 
 #else /* BUILTIN_IMAGELIB */
 
-  if (load_file(win->disp, ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win), path, pic_mod, &pixmap, NULL)) {
+  width = ACTUAL_WIDTH(win);
+  height = ACTUAL_HEIGHT(win);
+  if (load_file(win->disp, path, &width, &height, 0, pic_mod, &pixmap, NULL)) {
     return pixmap;
   } else {
     return None;
@@ -1506,7 +1526,7 @@ Pixmap ui_imagelib_get_transparent_background(ui_window_t *win, ui_picture_modif
   }
 
   if (!ui_picture_modifier_is_normal(pic_mod)) {
-    if (!modify_pixmap(win->disp, pixmap, pixmap, pic_mod)) {
+    if (!modify_pixmap(win->disp, pixmap, pixmap, width, height, pic_mod)) {
       XFreePixmap(win->disp->display, pixmap);
 
       return None;
@@ -1525,31 +1545,21 @@ Pixmap ui_imagelib_get_transparent_background(ui_window_t *win, ui_picture_modif
  *\param pixmap Returns an image pixmap for the old WM hint.
  *\param mask Returns a mask bitmap for the old WM hint.
  *\param width Pointer to the desired width. If *width is 0, the returned image
- *would not be scaled and *width would be overwritten by its width. "width" can
- *be NULL and the image would not be scaled and nothing would be returned in
- *this case.
- *\param height Pointer to the desired height. *height can be 0 and height can
- *be NULL(see "width" 's description)
+ *would not be scaled and *width would be overwritten by its width.
+ *\param height Pointer to the desired height. *height can be 0.
+ *(see "width" 's description)
  *
  *\return  Success => 1, Failure => 0
  */
 int ui_imagelib_load_file(ui_display_t *disp, char *path, u_int32_t **cardinal, Pixmap *pixmap,
-                          PixmapMask *mask, u_int *width, u_int *height) {
+                          PixmapMask *mask, u_int *width, u_int *height, int keep_aspect) {
   u_int dst_height, dst_width;
 #ifdef BUILTIN_IMAGELIB
   GdkPixbuf *pixbuf;
 #endif
 
-  if (!width) {
-    dst_width = 0;
-  } else {
-    dst_width = *width;
-  }
-  if (!height) {
-    dst_height = 0;
-  } else {
-    dst_height = *height;
-  }
+  dst_width = *width;
+  dst_height = *height;
 
 #if defined(BUILTIN_IMAGELIB) || defined(BUILTIN_SIXEL)
   if (!cardinal && strcasecmp(path + strlen(path) - 4, ".six") == 0 && dst_width == 0 &&
@@ -1562,7 +1572,7 @@ int ui_imagelib_load_file(ui_display_t *disp, char *path, u_int32_t **cardinal,
 
   if (path) {
     /* create a pixbuf from the file and create a cardinal array */
-    if (!(pixbuf = load_file(path, dst_width, dst_height, GDK_INTERP_BILINEAR))) {
+    if (!(pixbuf = load_file(path, dst_width, dst_height, keep_aspect, GDK_INTERP_BILINEAR))) {
 #ifdef DEBUG
       bl_warn_printf(BL_DEBUG_TAG "couldn't load pixbuf\n");
 #endif
@@ -1631,15 +1641,10 @@ int ui_imagelib_load_file(ui_display_t *disp, char *path, u_int32_t **cardinal,
     return 0;
   }
 
-  if (!load_file(disp, dst_width, dst_height, path, NULL, pixmap, mask)) {
+  if (!load_file(disp, path, &dst_width, &dst_height, keep_aspect, NULL, pixmap, mask)) {
     return 0;
   }
 
-  /* XXX Duplicated in load_file */
-  if (dst_width == 0 || dst_height == 0) {
-    get_drawable_size(disp->display, *pixmap, &dst_width, &dst_height);
-  }
-
   if (cardinal) {
     if (!(*cardinal = create_cardinals_from_file(path, dst_width, dst_height))) {
       goto error;
@@ -1648,11 +1653,8 @@ int ui_imagelib_load_file(ui_display_t *disp, char *path, u_int32_t **cardinal,
 
 #endif /* BUILTIN_IMAGELIB */
 
-  if (width && *width == 0) {
+  if (*width == 0 || *height == 0 || keep_aspect) {
     *width = dst_width;
-  }
-
-  if (height && *height == 0) {
     *height = dst_height;
   }
 
