@@ -15,7 +15,7 @@
 #include <X11/Xatom.h>
 #include <pobl/bl_debug.h>
 #include <pobl/bl_mem.h>    /* realloc/free */
-#include <pobl/bl_util.h>   /* K_MAX */
+#include <pobl/bl_util.h>   /* BL_MAX */
 #include <pobl/bl_unistd.h> /* bl_usleep */
 #include <pobl/bl_locale.h> /* bl_get_codeset() */
 
@@ -1380,8 +1380,12 @@ int ui_window_show(ui_window_t *win, int hint) {
     bl_debug_printf(BL_DEBUG_TAG " Group leader -> %x\n", wm_hints.window_group);
 #endif
 
-/* notify to window manager */
-#if 1
+    /*
+     * Notify to window manager.
+     *
+     * XXX If class_hint is set, lxde (0.99.2) doesn't show title bar for mlterm.
+     */
+#if 0
     XmbSetWMProperties(win->disp->display, win->my_window, win->app_name, win->app_name, argv, argc,
                        &size_hints, &wm_hints, &class_hint);
 #else
@@ -2066,8 +2070,8 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
 
       /* Minimum character size is regarded as w5 ui.h10. */
       if ((ev.xexpose.width * ev.xexpose.height) / 4 >=
-          (K_MAX(event->xexpose.width, 5) * K_MAX(event->xexpose.height, 10) +
-           K_MAX(next_ev.xexpose.width, 5) * K_MAX(next_ev.xexpose.height, 10)) /
+          (BL_MAX(event->xexpose.width, 5) * BL_MAX(event->xexpose.height, 10) +
+           BL_MAX(next_ev.xexpose.width, 5) * BL_MAX(next_ev.xexpose.height, 10)) /
               3) {
 /* Redrawing area is increased over 33.3% by this combination. */
 
@@ -3388,6 +3392,15 @@ void ui_window_set_input_focus(ui_window_t *win) {
   XSetInputFocus(win->disp->display, win->my_window, RevertToParent, CurrentTime);
 }
 
+void ui_window_flush(ui_window_t *win) {
+#if !defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
+  if (win->cairo_draw) {
+    /* calls cairo_flush() */
+    ui_window_cairo_draw_string32(win, NULL, NULL, 0, 0, NULL, 0);
+  }
+#endif
+}
+
 #ifdef DEBUG
 void ui_window_dump_children(ui_window_t *win) {
   int count;
