@@ -114,7 +114,6 @@ static u_int num_sessions = 0;
 
 #ifdef USE_WIN32API
 static HANDLE rd_ev;
-DWORD main_tid; /* XXX set in main(). */
 #endif
 
 static const char *cipher_list;
@@ -133,6 +132,8 @@ static int auto_reconnect;
 
 #ifdef USE_WIN32API
 
+static void (*trigger_pty_read)(void);
+
 static u_int __stdcall wait_pty_read(LPVOID thr_param) {
   u_int count;
   struct timeval tval;
@@ -167,9 +168,7 @@ static u_int __stdcall wait_pty_read(LPVOID thr_param) {
     }
 
     if (select(maxfd + 1, &read_fds, NULL, NULL, &tval) > 0) {
-      /* Exit GetMessage() in x_display_receive_next_event(). */
-      PostThreadMessage(main_tid, WM_APP, 0, 0);
-
+      (*trigger_pty_read)();
       WaitForSingleObject(rd_ev, INFINITE);
     }
 
@@ -473,6 +472,8 @@ static ssh_session_t *ssh_connect(const char *host, const char *port, const char
 
 #ifdef USE_WIN32API
       sprintf(path, "%s\\mlterm\\known_hosts", home);
+#elif defined(__ANDROID__)
+      sprintf(path, "%s/.mlterm/known_hosts", home);
 #else
       sprintf(path, "%s/.ssh/known_hosts", home);
 #endif
@@ -562,10 +563,19 @@ static ssh_session_t *ssh_connect(const char *host, const char *port, const char
     bl_debug_printf(BL_DEBUG_TAG " Unable to use ssh-agent.\n");
 #endif
 
-    if ((home = bl_get_home_dir()) && ((p = alloca(strlen(home) * 2 + 38)))) {
+    if ((home = bl_get_home_dir()) && ((p = alloca(strlen(home) * 2 + 36 /* __ANDROID__ */)))) {
+      /*
+       * man of libssh2_userauth_publickey_fromfile_ex()
+       * publickey - Path name of the public key file. (e.g. /etc/ssh/hostkey.pub)
+       * If libssh2 is built against OpenSSL, this option can be set to NULL.
+       *                                      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+       */
+#if 0
       if (!pubkey) {
 #ifdef USE_WIN32API
         sprintf(p, "%s\\mlterm\\id_rsa.pub", home);
+#elif defined(__ANDROID__)
+        sprintf(p, "%s/.mlterm/id_rsa.pub", home);
 #else
         sprintf(p, "%s/.ssh/id_rsa.pub", home);
 #endif
@@ -573,10 +583,13 @@ static ssh_session_t *ssh_connect(const char *host, const char *port, const char
         pubkey = p;
         p += (strlen(pubkey) + 1);
       }
+#endif
 
       if (!privkey) {
 #ifdef USE_WIN32API
         sprintf(p, "%s\\mlterm\\id_rsa", home);
+#elif defined(__ANDROID__)
+        sprintf(p, "%s/.mlterm/id_rsa", home);
 #else
         sprintf(p, "%s/.ssh/id_rsa", home);
 #endif
@@ -584,6 +597,7 @@ static ssh_session_t *ssh_connect(const char *host, const char *port, const char
         privkey = p;
       }
     } else {
+#if 0
       if (!pubkey) {
 #ifdef USE_WIN32API
         pubkey = "mlterm\\ssh_host_rsa_key.pub";
@@ -591,6 +605,7 @@ static ssh_session_t *ssh_connect(const char *host, const char *port, const char
         pubkey = "/etc/ssh/ssh_host_rsa_key.pub";
 #endif
       }
+#endif
 
       if (!privkey) {
 #ifdef USE_WIN32API
@@ -1174,7 +1189,7 @@ static void *
 
 #ifdef USE_WIN32API
       /* Exit GetMessage() in x_display_receive_next_event(). */
-      PostThreadMessage(main_tid, WM_APP, 0, 0);
+      (*trigger_pty_read)();
 #endif
     }
   }
@@ -1861,6 +1876,12 @@ static char *get_user_name(void) {
 
 /* --- global functions --- */
 
+#ifdef USE_WIN32API
+void vt_pty_set_pty_read_trigger(void (*func)(void)) {
+  trigger_pty_read = func;
+}
+#endif
+
 /*
  * Thread-safe.
  */
@@ -2007,6 +2028,13 @@ int vt_pty_ssh_scp_intern(vt_pty_t *pty, int src_is_remote, char *dst_path, char
 
     fstat(scp->local, &st);
 
+#if 1
+    if ((st.st_mode & 0700) == 0) {
+      /* XXX Avoid failure of libssh2_scp_send() */
+      st.st_mode |= 0400;
+    }
+#endif
+
     while (!(scp->remote = libssh2_scp_send(scp->pty_ssh->session->obj, dst_path, st.st_mode & 0777,
                                             (u_long)st.st_size)) &&
            libssh2_session_last_errno(scp->pty_ssh->session->obj) == LIBSSH2_ERROR_EAGAIN)
@@ -2070,7 +2098,7 @@ void vt_pty_ssh_set_keepalive_interval(u_int interval_sec) {
   keepalive_msec_left = keepalive_msec = interval_sec * 1000;
 }
 
-int vt_pty_ssh_keepalive(u_int spent_msec) {
+u_int vt_pty_ssh_keepalive(u_int spent_msec) {
   if (keepalive_msec_left <= spent_msec) {
     u_int count;
 
@@ -2083,7 +2111,7 @@ int vt_pty_ssh_keepalive(u_int spent_msec) {
     keepalive_msec_left -= spent_msec;
   }
 
-  return 1;
+  return keepalive_msec_left;
 }
 
 void vt_pty_ssh_set_use_x11_forwarding(void *session, int use) {
