@@ -131,6 +131,15 @@ static int insert_chars(vt_edit_t *edit, vt_char_t *ins_chars, u_int num_ins_cha
   for (count = 0; count < num_ins_chars; count++) {
     cols = vt_char_cols(&ins_chars[count]);
     if (cursor_col + filled_cols + cols > num_cols) {
+      /*
+       * ---+      ---+
+       *    |         |
+       * abcde  => abe|
+       */
+      if (filled_len > 0) {
+        vt_char_copy(&buffer[filled_len - 1], &ins_chars[num_ins_chars - 1]);
+      }
+
       break;
     }
 
@@ -217,11 +226,19 @@ static int horizontal_tabs(vt_edit_t *edit, u_int num, int is_forward) {
     vt_edit_goto_beg_of_line(edit);
   }
 
+  /*
+   * Compatible with rlogin 2.23.1.
+   *
+   * To be compatible with xterm-332, enclose by #if 0 ... #endif.
+   * (esctest: DECSETTests.test_DECSET_DECAWM_NoLineWrapOnTabWithLeftRightMargin)
+   */
+#if 1
   if (edit->cursor.col < edit->hmargin_beg) {
     vt_cursor_goto_by_col(&edit->cursor, edit->hmargin_beg, edit->cursor.row);
   } else if (edit->cursor.col > edit->hmargin_end) {
     vt_cursor_goto_by_col(&edit->cursor, edit->hmargin_end, edit->cursor.row);
   }
+#endif
 
   col = edit->cursor.col;
 
@@ -420,13 +437,8 @@ static int scroll_upward_region(vt_edit_t *edit, u_int size, int is_cursor_beg,
     vmargin_beg = edit->vmargin_beg;
   }
 
-  /*
-   * XXX
-   * CURSOR_IS_INSIDE_HMARGIN(edit) disables vim to scroll the right side of
-   * vertical splitted window.
-   */
   if (ignore_cursor_pos ||
-      (/* CURSOR_IS_INSIDE_HMARGIN(edit) && */
+      (CURSOR_IS_INSIDE_HMARGIN(edit) &&
        edit->cursor.row >= vmargin_beg && edit->cursor.row <= edit->vmargin_end)) {
     if (size > edit->vmargin_end - vmargin_beg + 1) {
       size = edit->vmargin_end - vmargin_beg + 1;
@@ -461,6 +473,18 @@ static int apply_relative_origin(vt_edit_t *edit, int *col, int *row, u_int *num
     if ((*col) + (*num_cols) > edit->hmargin_end + 1) {
       (*num_cols) = edit->hmargin_end + 1 - (*col);
     }
+  } else {
+    if ((*row) >= edit->model.num_rows || (*col) >= edit->model.num_cols) {
+      return 0;
+    }
+
+    if ((*row) + (*num_rows) > edit->model.num_rows) {
+      (*num_rows) = edit->model.num_rows - (*row);
+    }
+
+    if ((*col) + (*num_cols) > edit->model.num_cols) {
+      (*num_cols) = edit->model.num_cols - (*col);
+    }
   }
 
   return 1;
@@ -633,6 +657,14 @@ int vt_edit_resize(vt_edit_t *edit, u_int num_cols, u_int num_rows) {
 }
 
 int vt_edit_insert_chars(vt_edit_t *edit, vt_char_t *ins_chars, u_int num_ins_chars) {
+  /*
+   * edit->wraparound_ready_line is ignored.
+   * esctest: SMTests.test_SM_IRM_DoesNotWrapUnlessCursorAtMargin fails by this.
+   *
+   * XXX
+   * xterm-332, TeraTerm-4.95: Wraparound works if IRM is set.
+   * rlogin-2.23.1: Wraparound is disabled if IRM is set.
+   */
   reset_wraparound_checker(edit);
 
 #ifdef COMPAT_XTERM
@@ -753,6 +785,16 @@ int vt_edit_overwrite_chars(vt_edit_t *edit, vt_char_t *ow_chars, u_int num_ow_c
       vt_line_overwrite(line, edit->cursor.char_index, &buffer[beg], count - beg, cols);
 
       if (!edit->is_auto_wrap) {
+        /*
+         * ---+      ---+
+         *    |         |
+         * abcde  => abe|
+         * (esctest: DECSET_DECAWM_OffRespectsLeftRightMargin)
+         */
+        if (count > 0) {
+          vt_char_copy(&buffer[count - 1], &buffer[filled_len - 1]);
+        }
+
         break;
       }
 
@@ -833,6 +875,7 @@ int vt_edit_delete_cols(vt_edit_t *edit, u_int del_cols) {
   u_int buf_len;
   u_int filled_len;
   vt_line_t *cursor_line;
+  u_int num_filled_cols;
 
 #ifdef CURSOR_DEBUG
   vt_cursor_dump(&edit->cursor);
@@ -842,10 +885,10 @@ int vt_edit_delete_cols(vt_edit_t *edit, u_int del_cols) {
 
   cursor_line = CURSOR_LINE(edit);
 
-  /* XXX del_cols should be converted to del_chars */
-  if (edit->cursor.char_index + del_cols >= cursor_line->num_filled_chars) {
-    /* no need to overwrite */
+  num_filled_cols = vt_line_get_num_filled_cols(cursor_line);
 
+  if (!MARGIN_IS_ENABLED(edit) && edit->cursor.col + del_cols >= num_filled_cols) {
+    /* no need to overwrite */
     vt_edit_clear_line_to_right(edit); /* Considering BCE */
 
     return 1;
@@ -855,7 +898,7 @@ int vt_edit_delete_cols(vt_edit_t *edit, u_int del_cols) {
    * collecting chars after cursor line.
    */
 
-  buf_len = cursor_line->num_filled_chars;
+  buf_len = cursor_line->num_chars - edit->cursor.col;
 
   if ((buffer = vt_str_alloca(buf_len)) == NULL) {
 #ifdef DEBUG
@@ -914,42 +957,46 @@ int vt_edit_delete_cols(vt_edit_t *edit, u_int del_cols) {
     cols = vt_char_cols(vt_char_at(cursor_line, char_index++));
 
     if (MARGIN_IS_ENABLED(edit)) {
-      u_int count;
-      u_int copy_len;
-
       if (!CURSOR_IS_INSIDE_HMARGIN(edit)) {
         return 0;
       }
 
-      while (cols < del_cols && edit->cursor.col + cols < edit->hmargin_end + 1 &&
-             char_index < cursor_line->num_filled_chars) {
-        cols += vt_char_cols(vt_char_at(cursor_line, char_index++));
-      }
+      if (num_filled_cols > edit->hmargin_end + 1) {
+        u_int count;
+        u_int copy_len;
 
-      copy_len = 0;
-      while (edit->cursor.col + cols < edit->hmargin_end + 1 &&
-             char_index + copy_len < cursor_line->num_filled_chars) {
-        cols += vt_char_cols(vt_char_at(cursor_line, char_index + (copy_len++)));
-      }
+        while (cols < del_cols && edit->cursor.col + cols <= edit->hmargin_end) {
+          cols += vt_char_cols(vt_char_at(cursor_line, char_index++));
+        }
+        del_cols = cols;
 
-      vt_str_copy(buffer + filled_len, vt_char_at(cursor_line, char_index), copy_len);
-      filled_len += copy_len;
-      char_index += copy_len;
+        while (edit->cursor.col + (cols++) <= edit->hmargin_end) {
+          vt_char_copy(buffer + filled_len++, vt_char_at(cursor_line, char_index++));
+        }
 
-      for (count = 0; count < del_cols; count++) {
-        vt_char_copy(buffer + (filled_len++), edit->use_bce ? &edit->bce_ch : vt_sp_ch());
+        for (count = 0; count < del_cols; count++) {
+          vt_char_copy(buffer + (filled_len++), edit->use_bce ? &edit->bce_ch : vt_sp_ch());
+        }
+
+        copy_len = 0;
+        while (char_index + copy_len < cursor_line->num_filled_chars) {
+          vt_char_cols(vt_char_at(cursor_line, char_index + (copy_len++)));
+        }
+
+        vt_str_copy(buffer + filled_len, vt_char_at(cursor_line, char_index), copy_len);
+        filled_len += copy_len;
       }
     } else {
       while (cols < del_cols && char_index < cursor_line->num_filled_chars) {
         cols += vt_char_cols(vt_char_at(cursor_line, char_index++));
       }
+
+      vt_str_copy(buffer + filled_len, vt_char_at(cursor_line, char_index),
+                  cursor_line->num_filled_chars - char_index);
+      filled_len += (cursor_line->num_filled_chars - char_index);
     }
   }
 
-  vt_str_copy(buffer + filled_len, vt_char_at(cursor_line, char_index),
-              cursor_line->num_filled_chars - char_index);
-  filled_len += (cursor_line->num_filled_chars - char_index);
-
   if (filled_len > 0) {
     /*
      * overwriting.
@@ -1198,38 +1245,37 @@ void vt_edit_restore_protected_chars(vt_edit_t *edit, vt_protect_store_t *save)
 
 int vt_edit_set_vmargin(vt_edit_t *edit, int beg, int end) {
   /*
-   * for compatibility with xterm:
+   * If beg and end is -1, use default(full size of window).
+   * (see vt_parser.c)
    *
-   *   1. if beg and end is -1, use default.
-   *   2. if beg and end are smaller than 0, ignore the sequence.
-   *   3. if end is not larger than beg, ignore the sequence.
-   *   4. if beg and end are out of window, ignore the sequence.
-   *
-   *   (default = full size of window)
+   * For compatibility with xterm:
+   * 1. if beg and end are smaller than 0, ignore the sequence.
+   * 2. if end is not larger than beg, ignore the sequence.
+   * 3. if beg and end are out of window, ignore the sequence.
    */
 
-  if (beg == -1) {
-    beg = 0;
+  if (beg < 0) {
+    if (beg == -1) {
+      beg = 0;
+    } else {
+      return 0;
+    }
   }
 
-  if (end == -1) {
-    end = vt_model_end_row(&edit->model);
-  }
-
-  if (beg < 0 || end < 0) {
-    return 0;
+  if (end < 0) {
+    if (end == -1) {
+      end = vt_model_end_row(&edit->model);
+    } else {
+      return 0;
+    }
   }
 
   if (beg >= end) {
     return 0;
   }
 
-  if (beg >= edit->model.num_rows && end >= edit->model.num_rows) {
-    return 0;
-  }
-
   if (beg >= edit->model.num_rows) {
-    beg = vt_model_end_row(&edit->model);
+    return 0;
   }
 
   if (end >= edit->model.num_rows) {
@@ -1239,8 +1285,6 @@ int vt_edit_set_vmargin(vt_edit_t *edit, int beg, int end) {
   edit->vmargin_beg = beg;
   edit->vmargin_end = end;
 
-  vt_edit_goto(edit, 0, 0);
-
   return 1;
 }
 
@@ -1249,7 +1293,7 @@ void vt_edit_scroll_leftward(vt_edit_t *edit, u_int size) {
             edit->hmargin_end - edit->hmargin_beg + 1 - size,
             edit->vmargin_end - edit->vmargin_beg + 1,
             edit, edit->hmargin_beg, edit->vmargin_beg);
-  erase_area(edit, edit->hmargin_end - edit->hmargin_beg + 1 - size, edit->vmargin_beg,
+  erase_area(edit, edit->hmargin_end + 1 - size, edit->vmargin_beg,
              size, edit->vmargin_end - edit->vmargin_beg + 1);
 }
 
@@ -1343,7 +1387,7 @@ int vt_edit_scroll_downward(vt_edit_t *edit, u_int size) {
 }
 
 void vt_edit_set_use_hmargin(vt_edit_t *edit, int use) {
-  if (use <= 0) {
+  if (!use) {
     edit->use_margin = 0;
     edit->hmargin_beg = 0;
     edit->hmargin_end = edit->model.num_cols - 1;
@@ -1353,21 +1397,76 @@ void vt_edit_set_use_hmargin(vt_edit_t *edit, int use) {
 }
 
 int vt_edit_set_hmargin(vt_edit_t *edit, int beg, int end) {
-  if (edit->use_margin && 0 <= beg && beg < end && end < edit->model.num_cols) {
-    edit->hmargin_beg = beg;
-    edit->hmargin_end = end;
-
-    vt_edit_goto(edit, 0, 0);
-
-    return 1;
-  } else {
+  if (!edit->use_margin) {
+    /*
+     * The terminal only recognizes DECSLRM if vertical split screen mode (DECLRMM) is set.
+     * (see https://www.vt100.net/docs/vt510-rm/DECSLRM.html)
+     */
     return 0;
   }
+
+  /*
+   * If beg and end is -1, use default(full size of window).
+   * (see vt_parser.c)
+   *
+   * For compatibility with xterm:
+   * 1. if beg and end are smaller than 0, ignore the sequence.
+   * 2. if end is not larger than beg, ignore the sequence.
+   * 3. if beg and end are out of window, ignore the sequence.
+   */
+
+  if (beg < 0) {
+    if (beg == -1) {
+      beg = 0;
+    } else {
+      return 0;
+    }
+  }
+
+  if (end < 0) {
+    if (end == -1) {
+      end = edit->model.num_cols - 1;
+    } else {
+      return 0;
+    }
+  }
+
+  if (beg >= end) {
+    return 0;
+  }
+
+  if (beg >= edit->model.num_cols) {
+    return 0;
+  }
+
+  if (end >= edit->model.num_cols) {
+    end = edit->model.num_cols - 1;
+  }
+
+  edit->hmargin_beg = beg;
+  edit->hmargin_end = end;
+
+  return 1;
 }
 
 int vt_edit_forward_tabs(vt_edit_t *edit, u_int num) { return horizontal_tabs(edit, num, 1); }
 
-int vt_edit_backward_tabs(vt_edit_t *edit, u_int num) { return horizontal_tabs(edit, num, 0); }
+int vt_edit_backward_tabs(vt_edit_t *edit, u_int num) {
+#if 0
+  /* compat with xterm 332 CBT behavior (esctest: CHATests.test_CHA_IgnoresScrollRegion) */
+  int orig_hmargin_beg = edit->hmargin_beg;
+  int ret;
+
+  edit->hmargin_beg = 0;
+  ret = horizontal_tabs(edit, num, 0);
+  edit->hmargin_beg = orig_hmargin_beg;
+
+  return ret;
+#else
+  /* compat with RLogin 2.23.1 / Teraterm 4.95 */
+  return horizontal_tabs(edit, num, 0);
+#endif
+}
 
 void vt_edit_set_tab_size(vt_edit_t *edit, u_int tab_size) {
   int col;
@@ -1638,12 +1737,12 @@ int vt_edit_goto(vt_edit_t *edit, int col, int row) {
   return vt_cursor_goto_by_col(&edit->cursor, col, row);
 }
 
-void vt_edit_set_relative_origin(vt_edit_t *edit, int flag) {
-  edit->is_relative_origin = flag;
-}
-
-void vt_edit_set_auto_wrap(vt_edit_t *edit, int flag) {
-  edit->is_auto_wrap = flag;
+void vt_edit_set_last_column_flag(vt_edit_t *edit, int flag) {
+  if (flag) {
+    edit->wraparound_ready_line = CURSOR_LINE(edit);
+  } else {
+    reset_wraparound_checker(edit);
+  }
 }
 
 int vt_edit_restore_cursor(vt_edit_t *edit) {
@@ -1671,6 +1770,14 @@ void vt_edit_fill_area(vt_edit_t *edit, int code /* Unicode */, int is_protected
   vt_char_set(&ch, code,
               code <= 0x7f ? US_ASCII : ISO10646_UCS4_1, /* XXX biwidth is not supported. */
               0, 0,
+              /*
+               * xterm-332 and Tera Term 4.95 don't use BCE for DECALN(ESC#8), but use for DECFRA.
+               * rlogin-2.23.1 use BCE for both of them.
+               *
+               * (Test)
+               * echo -e "\x1b[0;33;44m\x1b[60;1;1;10;10\$x"
+               * echo -e "\x1b[0;33;44m\x1b#8"
+               */
               edit->use_bce ? vt_char_fg_color(&edit->bce_ch) : VT_FG_COLOR,
               edit->use_bce ? vt_char_bg_color(&edit->bce_ch) : VT_BG_COLOR,
               0, 0, 0, 0, is_protected);
@@ -1715,6 +1822,29 @@ void vt_edit_copy_area(vt_edit_t *src_edit, int src_col, int src_row, u_int num_
     if (dst_col + num_copy_cols > dst_edit->hmargin_end + 1) {
       num_copy_cols = dst_edit->hmargin_end + 1 - dst_col;
     }
+  } else {
+    if (src_row >= src_edit->model.num_rows ||
+        dst_row >= dst_edit->model.num_rows ||
+        src_col >= src_edit->model.num_cols ||
+        dst_col >= dst_edit->model.num_cols) {
+      return;
+    }
+
+    if (src_row + num_copy_rows > src_edit->model.num_rows) {
+      num_copy_rows = src_edit->model.num_rows - src_row;
+    }
+
+    if (dst_row + num_copy_rows > dst_edit->model.num_rows) {
+      num_copy_rows = dst_edit->model.num_rows - dst_row;
+    }
+
+    if (src_col + num_copy_cols > src_edit->model.num_cols) {
+      num_copy_cols = src_edit->model.num_cols - src_col;
+    }
+
+    if (dst_col + num_copy_cols > dst_edit->model.num_cols) {
+      num_copy_cols = dst_edit->model.num_cols - dst_col;
+    }
   }
 
   copy_area(src_edit, src_col, src_row, num_copy_cols, num_copy_rows,
@@ -1785,6 +1915,11 @@ void vt_edit_change_attr_area(vt_edit_t *edit, int col, int row, u_int num_cols,
     }
   }
 
+  /*
+   * XXX
+   * apply_relative_origin() adjusts arguments regarding edit->use_rect_attr_select as true
+   * all the time.
+   */
   if (!apply_relative_origin(edit, &col, &row, &num_cols, &num_rows)) {
     return;
   }
@@ -1831,6 +1966,66 @@ void vt_edit_change_attr_area(vt_edit_t *edit, int col, int row, u_int num_cols,
   }
 }
 
+u_int16_t vt_edit_get_checksum(vt_edit_t *edit, int col, int row, u_int num_cols, u_int num_rows) {
+  int count;
+  u_int16_t checksum;
+  vt_line_t *line;
+  int char_index;
+  u_int cols_rest;
+
+  if (!apply_relative_origin(edit, &col, &row, &num_cols, &num_rows)) {
+    return 0;
+  }
+
+  checksum = 0;
+  for (count = 0; count < num_rows; count++) {
+    if ((line = vt_edit_get_line(edit, row + count))) {
+      int end_char_index;
+      vt_char_t *ch;
+
+      if ((char_index = vt_convert_col_to_char_index(line, &cols_rest, col, BREAK_BOUNDARY)) >=
+          line->num_filled_chars) {
+        continue;
+      }
+
+      if (cols_rest > 0) {
+        char_index ++;
+        checksum --; /* 2nd column of full width character is 0xFFFF (compat with xterm) */
+      }
+
+      if ((end_char_index = vt_convert_col_to_char_index(line, &cols_rest,
+                                                         col + num_cols, BREAK_BOUNDARY)) >
+#if 0
+          line->num_filled_chars
+#else
+          vt_line_get_num_filled_chars_except_sp(line) /* compat with xterm */
+#endif
+          ) {
+#if 0
+        end_char_index = line->num_filled_chars;
+#else
+        end_char_index = vt_line_get_num_filled_chars_except_sp(line); /* compat with xterm */
+#endif
+      }
+
+      if (cols_rest > 0) {
+        end_char_index++;
+        checksum ++; /* col + num_cols - 1 is 1st column of full width character */
+      }
+
+      for(; char_index < end_char_index; char_index++) {
+        ch = vt_char_at(line, char_index);
+        checksum += vt_char_code(ch);
+        if (vt_char_cols(ch) == 2) {
+          checksum --; /* 2nd column of full width character is 0xFFFF (compat with xterm) */
+        }
+      }
+    }
+  }
+
+  return checksum;
+}
+
 void vt_edit_clear_size_attr(vt_edit_t *edit) {
   u_int count;
 
