@@ -16,6 +16,10 @@
 
 #define PTY_WR_BUFFER_SIZE 100
 
+#if 0
+#define ENABLE_OSC5379PICTURE
+#endif
+
 /*
  * Possible patterns are:
  *  NOT_USE_UNICODE_FONT(0x1)
@@ -67,10 +71,11 @@ typedef enum vt_locator_report_mode {
 
 typedef enum vt_alt_color_mode {
   ALT_COLOR_BOLD = 0x1,
-  ALT_COLOR_ITALIC = 0x2,
-  ALT_COLOR_UNDERLINE = 0x4,
-  ALT_COLOR_BLINKING = 0x8,
-  ALT_COLOR_CROSSED_OUT = 0x10,
+  ALT_COLOR_UNDERLINE = 0x2,
+  ALT_COLOR_BLINKING = 0x4,
+  ALT_COLOR_REVERSE = 0x8,
+  ALT_COLOR_ITALIC = 0x10,
+  ALT_COLOR_CROSSED_OUT = 0x20,
 
 } vt_alt_color_mode_t;
 
@@ -79,6 +84,7 @@ typedef enum vt_cursor_style {
   CS_UNDERLINE = 0x1,
   CS_BAR = 0x2,
   CS_BLINK = 0x4,
+  CS_BOX = 0x8,
 } vt_cursor_style_t;
 
 typedef struct vt_write_buffer {
@@ -104,9 +110,9 @@ typedef struct vt_read_buffer {
 typedef struct vt_xterm_event_listener {
   void *self;
 
-  void (*start)(void *);     /* called in *visual* context. (Note that not logical) */
-  void (*stop)(void *);      /* called in visual context. */
-  void (*interrupt)(void *); /* called in visual context. */
+  void (*start)(void *);             /* called in *visual* context. (Note that not logical) */
+  void (*stop)(void *);                                        /* called in visual context. */
+  void (*interrupt)(void *);                                   /* called in visual context. */
 
   void (*resize)(void *, u_int, u_int, int);                   /* called in visual context. */
   void (*reverse_video)(void *, int);                          /* called in visual context. */
@@ -118,16 +124,20 @@ typedef struct vt_xterm_event_listener {
   int (*im_is_active)(void *);                                 /* called in logical context. */
   void (*switch_im_mode)(void *);                              /* called in logical context. */
   void (*set_selection)(void *, vt_char_t *, u_int, u_char *); /* called in logical context. */
-  int (*get_window_size)(void *, u_int *, u_int *);            /* called in logical context. */
+  void (*get_display_size)(void *, u_int *, u_int *);          /* called in logical context. */
+  void (*get_window_size)(void *, u_int *, u_int *);           /* called in logical context. */
   int (*get_rgb)(void *, u_int8_t *, u_int8_t *, u_int8_t *,
-                 vt_color_t); /* called in logical context. */
-  vt_char_t *(*get_picture_data)(void *, char *, int *, int *,
-                                 u_int32_t **);                 /* called in logical context. */
-  int (*get_emoji_data)(void *, vt_char_t *, vt_char_t *);      /* called in logical context. */
-  void (*show_sixel)(void *, char *);                           /* called in logical context. */
-  void (*add_frame_to_animation)(void *, char *, int *, int *); /* called in logical context. */
-  void (*hide_cursor)(void *, int);                             /* called in logical context. */
-  int (*check_iscii_font)(void *, ef_charset_t);
+                 vt_color_t);                                  /* called in logical context. */
+  vt_char_t *(*get_picture_data)(void *, char *, int *, int *, int *, int *,
+                                 u_int32_t **, int, int);      /* called in logical context. */
+  int (*get_emoji_data)(void *, vt_char_t *, vt_char_t *);     /* called in logical context. */
+  void (*show_tmp_picture)(void *, char *);                    /* called in logical context. */
+#ifdef ENABLE_OSC5379PICTURE
+  void (*add_frame_to_animation)(void *, char *, int *, int *);/* called in logical context. */
+#endif
+  void (*hide_cursor)(void *, int);                            /* called in logical context. */
+  int (*check_iscii_font)(void *, ef_charset_t);               /* called in logical context. */
+  void (*lock_keyboard)(void *, int);                          /* called in logical context. */
 
 } vt_xterm_event_listener_t;
 
@@ -164,8 +174,11 @@ typedef struct vt_storable_states {
   int is_reversed : 1;
   int is_blinking : 1;
   int is_invisible : 1;
+  int is_protected : 1;
+  int is_relative_origin : 1;
+  int last_column_flag : 1;
 
-  /* vt_line_style_t */ int8_t line_style;
+  /* vt_line_style_t */ int line_style : 7;
   vt_color_t fg_color;
   vt_color_t bg_color;
   ef_charset_t cs;
@@ -309,6 +322,8 @@ void vt_set_unicode_noconv_areas(char *areas);
 
 void vt_set_full_width_areas(char *areas);
 
+void vt_set_half_width_areas(char *areas);
+
 void vt_set_use_ttyrec_format(int use);
 
 #ifdef USE_LIBSSH2
@@ -361,7 +376,7 @@ int vt_parser_write_loopback(vt_parser_t *vt_parser, const u_char *buf, size_t l
 /* Must be called in visual context. */
 int vt_parser_show_message(vt_parser_t *vt_parser, char *msg);
 
-#if defined(__ANDROID__) || defined(__APPLE__)
+#if 1 /* defined(__ANDROID__) || defined(__APPLE__) || defined(USE_SDL2) */
 /* Must be called in visual context. */
 int vt_parser_preedit(vt_parser_t *vt_parser, const u_char *buf, size_t len);
 #endif
@@ -389,9 +404,6 @@ void vt_init_encoding_conv(vt_parser_t *vt_parser);
 
 #define vt_parser_is_using_char_combining(vt_parser) ((vt_parser)->use_char_combining)
 
-#define vt_parser_set_use_multi_col_char(vt_parser, use) \
-  ((vt_parser)->use_multi_col_char = (use))
-
 #define vt_parser_is_using_multi_col_char(vt_parser) ((vt_parser)->use_multi_col_char)
 
 #define vt_parser_get_mouse_report_mode(vt_parser) ((vt_parser)->mouse_mode)
