@@ -12,10 +12,14 @@
 
 /*
  * defined(__CYGWIN__) is not to link libpthread to mlterm for now.
+ *
  * OPEN_PTY_SYNC is defined in java/Makefile.in
+ *
+ * Note that bl_dialog() in open_pty() in vt_term.c might cause segfault if
+ * OPEN_PTY_ASYNC is defined on platforms other than WIN32GUI,
  */
 #if (defined(USE_WIN32API) && !defined(OPEN_PTY_SYNC)) || \
-    (defined(HAVE_PTHREAD) && defined(__CYGWIN__))
+    (defined(HAVE_PTHREAD) && (defined(__CYGWIN__) || defined(__MSYS__)))
 #define OPEN_PTY_ASYNC
 #endif
 
@@ -87,7 +91,7 @@ void vt_pty_ssh_set_cipher_list(const char *list);
 
 void vt_pty_ssh_set_keepalive_interval(u_int interval_sec);
 
-int vt_pty_ssh_keepalive(u_int spent_msec);
+u_int vt_pty_ssh_keepalive(u_int spent_msec);
 
 void vt_pty_ssh_set_use_x11_forwarding(void *session, int use_x11_forwarding);
 
@@ -100,4 +104,8 @@ int vt_pty_ssh_send_recv_x11(int idx, int bidirection);
 void vt_pty_ssh_set_use_auto_reconnect(int flag);
 #endif
 
+#ifdef USE_WIN32API
+void vt_pty_set_pty_read_trigger(void (*func)(void));
+#endif
+
 #endif
