@@ -8,16 +8,13 @@
 #include <pobl/bl_debug.h>
 #include <pobl/bl_mem.h>  /* malloc/free */
 #include <pobl/bl_str.h>  /* strdup */
-#include <pobl/bl_util.h> /* K_MIN */
+#include <pobl/bl_util.h> /* BL_MIN */
 
 #include "vt_char_encoding.h"
 #include "vt_str_parser.h"
 
 #define ROW_IN_LOGS(screen, row) (vt_get_num_logged_lines(&(screen)->logs) + row)
 
-/* Not contains the first page. */
-#define MAX_PAGE_ID 8
-
 #if 1
 #define EXIT_BS_AT_BOTTOM
 #endif
@@ -306,7 +303,7 @@ static int reverse_or_restore_color(vt_screen_t *screen, /* visual */
 
     if (vt_line_is_rtl(line)) {
       if (row > beg_row || beg_char_index >= size_except_sp) {
-        beg_char_index = K_MAX(size_except_sp, 1) - 1;
+        beg_char_index = BL_MAX(size_except_sp, 1) - 1;
       } else if (beg_char_index < beg_regarding_rtl) {
         goto next_line;
       }
@@ -358,7 +355,7 @@ static int reverse_or_restore_color(vt_screen_t *screen, /* visual */
     beg_regarding_rtl = vt_line_beg_char_index_regarding_rtl(line);
 
     if (vt_line_is_rtl(line)) {
-      beg_char_index = K_MAX(size_except_sp, 1) - 1;
+      beg_char_index = BL_MAX(size_except_sp, 1) - 1;
     } else {
       beg_char_index = beg_regarding_rtl;
     }
@@ -368,14 +365,14 @@ static int reverse_or_restore_color(vt_screen_t *screen, /* visual */
 
   if (vt_line_is_rtl(line)) {
     if (end_char_index < size_except_sp) {
-      for (char_index = K_MAX(end_char_index, beg_regarding_rtl); char_index <= beg_char_index;
+      for (char_index = BL_MAX(end_char_index, beg_regarding_rtl); char_index <= beg_char_index;
            char_index++) {
         (*func)(line, char_index);
       }
     }
   } else {
     if (end_char_index >= beg_regarding_rtl) {
-      for (char_index = beg_char_index; char_index < K_MIN(end_char_index + 1, size_except_sp);
+      for (char_index = beg_char_index; char_index < BL_MIN(end_char_index + 1, size_except_sp);
            char_index++) {
         (*func)(line, char_index);
       }
@@ -476,7 +473,7 @@ static u_int check_or_copy_region(
 
     if (vt_line_is_rtl(line)) {
       if (row > beg_row || beg_char_index >= size_except_sp) {
-        beg_char_index = K_MAX(size_except_sp, 1) - 1;
+        beg_char_index = BL_MAX(size_except_sp, 1) - 1;
       } else if (beg_char_index < beg_regarding_rtl) {
         goto next_line;
       }
@@ -550,7 +547,7 @@ static u_int check_or_copy_region(
     beg_regarding_rtl = vt_line_beg_char_index_regarding_rtl(line);
 
     if (vt_line_is_rtl(line)) {
-      beg_char_index = K_MAX(size_except_sp, 1) - 1;
+      beg_char_index = BL_MAX(size_except_sp, 1) - 1;
     } else {
       beg_char_index = beg_regarding_rtl;
     }
@@ -1499,7 +1496,7 @@ int vt_screen_search_find(vt_screen_t *screen,
     if (backward) {
       vt_str_parser_set_str(parser, line_str, (*beg_row != screen->search->row)
                                                   ? line_len
-                                                  : K_MIN(*beg_char_index + 1, line_len));
+                                                  : BL_MIN(*beg_char_index + 1, line_len));
       *beg_char_index = 0;
     } else {
       if (*beg_row != screen->search->row) {
@@ -1836,7 +1833,11 @@ int vt_screen_go_downward(vt_screen_t *screen, u_int size) {
 
 int vt_screen_use_normal_edit(vt_screen_t *screen) {
   if (screen->edit != &screen->normal_edit) {
+    int col = screen->edit->cursor.col;
+    int row = screen->edit->cursor.row;
+
     change_edit(screen, &screen->normal_edit);
+    vt_edit_goto(screen->edit, col, row);
 
     return 1;
   } else {
@@ -1846,10 +1847,11 @@ int vt_screen_use_normal_edit(vt_screen_t *screen) {
 
 int vt_screen_use_alternative_edit(vt_screen_t *screen) {
   if (screen->edit != &screen->alt_edit) {
-    change_edit(screen, &screen->alt_edit);
+    int col = screen->edit->cursor.col;
+    int row = screen->edit->cursor.row;
 
-    vt_screen_goto_home(screen);
-    vt_screen_clear_below(screen);
+    change_edit(screen, &screen->alt_edit);
+    vt_edit_goto(screen->edit, col, row);
 
     return 1;
   } else {
@@ -1965,6 +1967,21 @@ void vt_screen_copy_area(vt_screen_t *screen, int src_col, int src_row, u_int nu
   }
 }
 
+u_int16_t vt_screen_get_checksum(vt_screen_t *screen, int col, int row,
+                             u_int num_cols, u_int num_rows, int page) {
+  vt_edit_t *edit;
+
+  if (page > MAX_PAGE_ID) {
+    page = MAX_PAGE_ID;
+  }
+
+  if ((edit = get_edit(screen, page))) {
+    return vt_edit_get_checksum(edit, col, row, num_cols, num_rows);
+  }
+
+  return 0;
+}
+
 void vt_screen_enable_blinking(vt_screen_t *screen) { screen->has_blinking_char = 1; }
 
 int vt_screen_write_content(vt_screen_t *screen, int fd, ef_conv_t *conv, int clear_at_end,
@@ -2104,6 +2121,15 @@ void vt_screen_set_use_status_line(vt_screen_t *screen, int use) {
       screen->main_edit = screen->edit;
       screen->has_status_line = 1;
       vt_edit_set_modified_all(screen->status_edit);
+      /*
+       * XXX
+       * I don't know whether the cursor goes to home position or not,
+       * but vttest 11.2.6.2.1 seems to assume it.
+       * (https://vt100.net/docs/vt510-rm/DECSSDT.html doesn't describe it.)
+       */
+#if 1
+      vt_edit_goto(screen->status_edit, 0, 0);
+#endif
 
       vt_screen_resize(screen, vt_edit_get_cols(screen->main_edit),
                        vt_edit_get_rows(screen->main_edit));
