@@ -11,6 +11,8 @@
 #include "vt_logs.h"
 #include "vt_logical_visual.h"
 
+#define MAX_PAGE_ID 8
+
 typedef struct vt_screen_event_listener {
   void *self;
 
@@ -334,8 +336,17 @@ int vt_screen_go_downward(vt_screen_t *screen, u_int size);
 
 #define vt_screen_is_auto_wrap(screen) vt_edit_is_auto_wrap((screen)->edit)
 
+#define vt_screen_set_last_column_flag(screen, flag) \
+  vt_edit_set_last_column_flag((screen)->edit, flag)
+
+#define vt_screen_get_last_column_flag(screen) vt_edit_get_last_column_flag((screen)->edit)
+
+#define vt_screen_get_bce_ch(screen) vt_edit_get_bce_ch((screen)->edit)
+
 #define vt_screen_save_cursor(screen) vt_edit_save_cursor((screen)->edit)
 
+#define vt_screen_saved_cursor_to_home(screen) vt_edit_saved_cursor_to_home((screen)->edit)
+
 #define vt_screen_restore_cursor(screen) vt_edit_restore_cursor((screen)->edit)
 
 int vt_screen_cursor_visible(vt_screen_t *screen);
@@ -382,6 +393,9 @@ void vt_screen_copy_area(vt_screen_t *screen, int src_col, int src_row, u_int nu
   vt_edit_change_attr_area((screen)->edit, col, row, num_cols, num_rows,        \
                            vt_char_reverse_attr, attr)
 
+u_int16_t vt_screen_get_checksum(vt_screen_t *screen, int col, int row,
+                             u_int num_cols, u_int num_rows, int page);
+
 #define vt_screen_set_use_rect_attr_select(screen, use) \
   vt_edit_set_use_rect_attr_select((screen)->edit, use)
 
