@@ -7,6 +7,7 @@
 #include <pobl/bl_str.h> /* strdup */
 #include <pobl/bl_sig_child.h>
 #include <pobl/bl_path.h> /* bl_parse_uri */
+#include <pobl/bl_dialog.h>
 
 #include "vt_pty.h"
 #include "vt_parser.h"
@@ -190,6 +191,8 @@ static void *
 
     vt_term_plug_pty(args->term, pty);
   } else {
+    bl_dialog(BL_DIALOG_ALERT, "Failed to open pty");
+
     args->term->return_special_pid = 1;
     bl_trigger_sig_child(-10);
     args->term->return_special_pid = 0;
@@ -400,9 +403,7 @@ int vt_term_open_pty(vt_term_t *term, const char *cmd_path, char **argv, char **
       if (!(pty = vt_pty_new(cmd_path, argv, env, host, work_dir, pass, pubkey, privkey,
                              vt_screen_get_logical_cols(term->screen),
                              vt_screen_get_logical_rows(term->screen), width_pix, height_pix))) {
-#ifdef DEBUG
-        bl_warn_printf(BL_DEBUG_TAG " vt_pty_new failed.\n");
-#endif
+        bl_dialog(BL_DIALOG_ALERT, "Failed to open pty");
 
         return 0;
       }
