@@ -161,7 +161,7 @@ size_t vt_term_write(vt_term_t *term, u_char *buf, size_t len);
 /* Must be called in visual context. */
 #define vt_term_show_message(term, msg) vt_parser_show_message((term)->parser, msg)
 
-#if defined(__ANDROID__) || defined(__APPLE__)
+#if 1 /* defined(__ANDROID__) || defined(__APPLE__) || defined(USE_SDL2) */
 /* Must be called in visual context. */
 #define vt_term_preedit(term, buf, len) vt_parser_preedit((term)->parser, buf, len)
 #endif
@@ -261,6 +261,8 @@ void vt_term_enter_backscroll_mode(vt_term_t *term);
   vt_screen_get_word_region((term)->screen, beg_char_index, beg_row, end_char_index, end_row, \
                             base_char_index, base_row)
 
+#define vt_term_get_bce_ch(term) vt_screen_get_bce_ch((term)->screen)
+
 #define vt_term_write_content(term, fd, conv, clear_at_end, area) \
   vt_screen_write_content(term->screen, fd, conv, clear_at_end, area)
 
